///////////////////////////////////////////////////////////////////////
//
//  File				:	containers.h
//  Classes				:	CDictionary
//							CHash
//							CTrie
//							CList
//							CMemPool
//							CMemStack
//  Description			:	Misc template data structures
//
//
//
//	11/28/2001
//
////////////////////////////////////////////////////////////////////////

#ifndef CONTAINERS_H
#define CONTAINERS_H

#include "str.h"

///////////////////////////////////////////////////////////////////////
// Class				:	CDictionary
// Description			:	Pure virtual parent class for dictionary
// Comments				:
// Date last edited		:	11/28/2001
template <class keyType,class valType> class CDictionary  {
public:
						CDictionary()	{	}
	virtual				~CDictionary()	{	}

						// inserts an entity into the dictionary
	virtual	void		insert(keyType,valType)				=	NULL;
						// return 1 if the entity is found in the dictionary
	virtual	int			find(keyType,valType *)				=	NULL;
						// return 1 if the entity is found in the dictionary and erase it
	virtual	int			erase(keyType,valType *val=NULL)	=	NULL;
						// Delete all the values stored in the dictionary
	virtual	void		destroy()							=	NULL;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CHash
// Description			:	Hash table
// Comments				:
// Date last edited		:	11/28/2001
template <class keyType,class valType>	class CHash : public CDictionary<keyType,valType> {
	// bucket class
	class CHashBucket {
		public:
			keyType		key;
			valType		val;
			CHashBucket	*next;
	};

public:
	// Constructor
	CHash(int nb = 50)		{
		int			i;
		nBuckets	=	nb;
		buckets		=	new CHashBucket*[nBuckets];
		for (i=0;i<nBuckets;i++)
			buckets[i]	=	NULL;
	}

	// Destructor
	~CHash() {
		CHashBucket	*h;
		int			i;

		for (i=0;i<nBuckets;i++) {
			h	=	buckets[i];

			if (h != NULL) {
				CHashBucket	*hn	=	h->next;

				delete h;

				h = hn;
			}
		}


		delete [] buckets;
	}

				
	void		insert(keyType k,valType v)	{
		int								index	=	key(k);
		CHashBucket						*h		=	new CHashBucket;

		h->key			=	k;
		h->val			=	v;
		h->next			=	buckets[index];
		buckets[index]	=	h;
	}

	int			find(keyType k,valType *v=NULL) {
		int								index	=	key(k);
		CHashBucket						*h;

		h	=	buckets[index];
		while(h != NULL) {
			if (keycmp(h->key,k)) {
				if (v != NULL)
					(*v)	=	h->val;
				return TRUE;
			}
		
			h	=	h->next;
		}

		if (v != NULL)			
			(*v)	=	NULL;

		return FALSE;
	}
						
	int			erase(keyType k,valType *v=NULL) {
		int								index	=	key(k);
		CHashBucket						*h;
		CHashBucket						*h1;

		h1	=	NULL;
		h	=	buckets[index];
		while(h != NULL) {
			if (keycmp(h->key,k)) {
				if (v != NULL)
					(*v)	=	h->val;

				if (h1 == NULL) buckets[index]	=	h->next;
				else h1->next = h->next;
					
				delete h;

				return TRUE;
			}

			h1	=	h;
			h	=	h->next;
		}

		if (v != NULL)
			(*v)	=	NULL;

		return FALSE;
	}

	void	destroy() {
		CHashBucket	*h;
		int			i;

		for (i=0;i<nBuckets;i++) {
			h	=	buckets[i];

			if (h != NULL) {
				CHashBucket	*hn	=	h->next;

				delete h->val;
				delete h;

				h = hn;
			}
		}

	}


	int					nBuckets;
	CHashBucket			**buckets;
private:
	int					key(char *s) {
		int l	=	slen(s);
		int	i;
		int	index	=	0;

		for (i=0;i<l;i++)
			index	+=	s[i]*s[i];

		index	%=	nBuckets;
		return index;
	}

	int					key(int s) {
		int	index	=	((s*s) + s) % nBuckets;

		return index;
	}

	int					keycmp(char *s1,char *s2) {
		if (scmp(s1,s2) == 0) return TRUE;
		return FALSE;
	}

	int					keycmp(int s1,int s2) {
		if (s1 == s2) return TRUE;
		return FALSE;
	}
};

///////////////////////////////////////////////////////////////////////
// Class				:	CTrie
// Description			:	Trie class
// Comments				:
// Date last edited		:	11/28/2001
const	int		TRIE_NODE_LEAF				=	1;	

template <class valType> class CTrie : public CDictionary<char *,valType> {
	class CTrieLeaf {
		public:
		CTrieLeaf(char *k,valType v) {
			key		=	k;
			val		=	v;
		}

		~CTrieLeaf() {
		}
	
		valType	val;
		char	*key;
	};

	class CTrieNode {
	public:
		CTrieNode() {
			int i;

			for (i=0;i<128;i++) pointers[i]	=	NULL;
		}

		~CTrieNode() {
			int i;

			for (i=0;i<128;i++) {
				if (pointers[i] != NULL) {
					if (((unsigned int) pointers[i]) & TRIE_NODE_LEAF)
						delete (CTrieLeaf *) (((unsigned int) pointers[i]) ^ TRIE_NODE_LEAF);
					else delete pointers[i];
				}
			}
		}

		CTrieNode	*pointers[128];
	};


	public:
		CTrie()	{
			root	=	NULL;
		}

		virtual	~CTrie() {
			if (root != NULL) delete root;
		}

		void	insert(char *key,valType val) {
			CTrieNode			*currentNode,*prevNode;
			int					index;

			prevNode	=	NULL;
			currentNode	=	root;

			for (index = 0; index <= (int) strlen(key); index++) {
				if (currentNode == NULL) {
					if (prevNode == NULL) {
						root	=	(CTrieNode *) (((unsigned int) new CTrieLeaf(&key[index],val)) | TRIE_NODE_LEAF); 
					} else {
						prevNode->pointers[key[index-1]]	=	(CTrieNode *) (((unsigned int) new CTrieLeaf(&key[index],val)) | TRIE_NODE_LEAF); 
					}

					return;
				} else if (((unsigned int) currentNode) & TRIE_NODE_LEAF) {
					CTrieNode	*oLeaf	=	currentNode;
					CTrieLeaf	*cLeaf	=	(CTrieLeaf *) (((unsigned int) currentNode) ^ TRIE_NODE_LEAF);

					if ((cLeaf->key[0] == '\0') && (key[index] == '\0')) {
						return;
					}

					currentNode								=	new CTrieNode;
					currentNode->pointers[cLeaf->key[0]]	=	oLeaf;

					if (cLeaf->key[0] == '\0') {
						cLeaf->key	=	NULL;
					} else {
						cLeaf->key	=	&cLeaf->key[1];
					}

					if (prevNode == NULL)	root = currentNode;
					else					prevNode->pointers[key[index-1]] = currentNode;
					
					insert(key,val);
					return;
				} else {
					prevNode	=	currentNode;
					currentNode	=	currentNode->pointers[key[index]];
				}
			}

			if (prevNode == NULL) {
				root	=	(CTrieNode *) (((unsigned int) new CTrieLeaf(&key[index],val)) | TRIE_NODE_LEAF); 
			} else {
				prevNode->pointers[key[index-1]]	=	(CTrieNode *) (((unsigned int) new CTrieLeaf(&key[index],val)) | TRIE_NODE_LEAF); 
			}
		}

		int		find(char *key,valType *val) {
			int					index;
			CTrieNode			*cNode;
			CTrieLeaf			*cLeaf;

			cNode	=	root;

			for (index=0;index <= (int) strlen(key); index++) {
				if (cNode == NULL) return 0;

				if (((unsigned int) cNode) & TRIE_NODE_LEAF) {
					cLeaf	=	(CTrieLeaf *) (((unsigned int) cNode) ^ TRIE_NODE_LEAF);
					if (cLeaf->key == NULL) {
						return FALSE;
					} else {
						if (scmp(&key[index],cLeaf->key) == 0) {
							if (val != NULL) (*val) = cLeaf->val;
							return TRUE;
						} else return FALSE;
					}
				} else {
					cNode	=	cNode->pointers[key[index]];
				}
			}
			if (cNode == NULL) return 0;

			if (((unsigned int) cNode) & TRIE_NODE_LEAF) {
				cLeaf	=	(CTrieLeaf *) (((unsigned int) cNode) ^ TRIE_NODE_LEAF);
				if (cLeaf->key == NULL) {
					if (val != NULL) (*val) = cLeaf->val;
					return TRUE;
				} else {
					if (scmp(&key[index],cLeaf->key) == 0) {
						if (val != NULL) (*val) = cLeaf->val;
						return 1;
					}
				}
			}

			return 0;
		}

		int	erase(char *key,valType *val=NULL) {
			int					index;
			CTrieNode			*cNode,*pNode;
			CTrieLeaf			*cLeaf;

			pNode	=	NULL;
			cNode	=	root;

			for (index=0;index <= (int) strlen(key); index++) {
				if (cNode == NULL) return 0;

				if (((unsigned int) cNode) & TRIE_NODE_LEAF) {
					cLeaf	=	(CTrieLeaf *) (((unsigned int) cNode) ^ TRIE_NODE_LEAF);
					if (cLeaf->key == NULL) {
						return FALSE;
					} else {
						if (scmp(&key[index],cLeaf->key) == 0) {
							if (pNode == NULL) root = NULL;
							else pNode->pointers[key[index-1]] = NULL;

							if (val != NULL)
								(*val)	=	cLeaf->val;

							delete cLeaf;
							return 1;
						}
					}
				} else {
					pNode	=	cNode;
					cNode	=	cNode->pointers[key[index]];
				}
			}
			return 0;

		}

		void	destroy() {
			if (root != NULL)	destroyNode(root);
			root	=	NULL;
			delete this;
		}

		CTrieNode		*root;

private:
	void	destroyNode(CTrieNode *cNode) {
		int	i;

		if ((unsigned int) cNode & TRIE_NODE_LEAF) {
			CTrieLeaf	*cLeaf	=	(CTrieLeaf *) ((unsigned int) cNode ^ TRIE_NODE_LEAF);
			delete cLeaf->val;
			delete cLeaf;
		} else {
			for (i=0;i<128;i++) {
				if (cNode->pointers[i] != NULL) {
					destroyNode(cNode->pointers[i]);
					cNode->pointers[i]	=	NULL;
				}
			}
			delete cNode;
		}

		
	}
};

///////////////////////////////////////////////////////////////////////
// Class				:	CList
// Description			:	List class
// Comments				:
// Date last edited		:	11/28/2001
template <class T> class CList  {
public:

			CList(int nb=10,int ni=50) {
				int				i;

				maxBunk			=	nb;
				bunkStep		=	nb;
				itemsPerbunk	=	ni;
				lastBunk		=	-1;
				lastItem		=	-1;
				currentItem		=	-1;
				currentBunk		=	-1;

				storage			=	new T*[maxBunk];
				for (i=0;i<maxBunk;i++) storage[i] = NULL;
			}

			CList(CList<T>	*a) {
				int				i;

				maxBunk			=	a->maxBunk;
				bunkStep		=	a->bunkStep;
				itemsPerbunk	=	a->itemsPerbunk;
				lastBunk		=	a->lastBunk;
				lastItem		=	a->lastItem;
				currentItem		=	a->currentItem;
				currentBunk		=	a->currentBunk;

				storage			=	new T*[maxBunk];
				for (i=0;i<maxBunk;i++) {
					if (a->storage[i] != NULL) {
						int		j;
						storage[i] = new T[itemsPerbunk];
						for (j=0;j<itemsPerbunk;j++)
							storage[i][j] = a->storage[i][j];
					} else
						storage[i] = NULL;
				}
			}


			~CList() {
				int i;

				for (i=0;i<maxBunk;i++) {
					if (storage[i] != NULL) delete [] storage[i];
				}

				delete [] storage;
			}

		void	destroy() {
				int b,i;

				if (lastItem == -1) return;

				for (b=0;b<=lastBunk;b++) {
					if (b != lastBunk) 
						for (i=0;i<itemsPerbunk;i++)
							delete storage[b][i];
					else 
						for (i=0;i<=lastItem;i++)
							delete storage[b][i];
			}

				}
	void	push(T a) {
				if (lastItem == -1) {
					if (storage[0] == NULL) storage[0] = new T[itemsPerbunk];

					lastBunk = 0;
					lastItem = 0;
				} else {
					lastItem++;
					if (lastItem == itemsPerbunk) {
						lastBunk++;
						if (lastBunk == maxBunk) {
							int		i;
							T	**news = new T*[maxBunk+bunkStep];

							for (i=0;i<maxBunk;i++)
								news[i] = storage[i];
							for (;i<maxBunk+bunkStep;i++)
								news[i] = NULL;

							delete [] storage;
							storage = news;
							maxBunk += bunkStep;
						}

						if (storage[lastBunk] == NULL)
							storage[lastBunk] = new T[itemsPerbunk];
				
						lastItem = 0;
					}
				}

				storage[lastBunk][lastItem] = a;
			}

	T		pop() {
				T tmp;

				if (lastItem == -1) return (T) 0;

				tmp = storage[lastBunk][lastItem];

				lastItem--;
				if (lastItem == -1) {
					lastBunk--;
					if (lastBunk >= 0) lastItem = itemsPerbunk-1;
				}

				return tmp;
			}

	T		first() {

				if (lastItem == -1) return (T) 0;

				currentBunk = 0;
				currentItem = 0;

				return storage[currentBunk][currentItem];
			}

	T		last() {
				if (lastItem == -1) return NULL;

				currentBunk =	lastBunk;
				currentItem	=	lastItem;

				return storage[currentBunk][currentItem];
			}

	T		current() {
				return storage[currentBunk][currentItem];
			}

	T		next() {
				currentItem++;

				if (currentBunk == lastBunk) {
					if (currentItem > lastItem) {
						currentItem--;

						return NULL;
					}
				} else {
					if (currentItem == itemsPerbunk) {
						currentBunk++;
						currentItem = 0;
					}
				}

				return storage[currentBunk][currentItem];
			}

	T		prev() {
				currentItem--;

				if (currentItem == -1) {
					if (currentBunk == 0) {
						currentItem++;
						return NULL;
					}
					currentBunk--;
					currentItem = itemsPerbunk-1;
				}

				return storage[currentBunk][currentItem];
			}

	int		numItems() {
				if (lastBunk == -1) return 0;

				return (lastBunk*itemsPerbunk + lastItem+1);
			}

	T		random() {
				int	r		=	rand();
				int	bunk	=	r % (lastBunk + 1);
				int	item	=	r % (lastItem + 1);

				return storage[bunk][item];
			}

	T		ditch() {
				lastItem--;
				if (lastItem == -1) {
					if (lastBunk == 0) {
						lastItem++;
						return NULL;
					}
					lastBunk--;
					lastItem = itemsPerBunk-1;
				}

				storage[currentBunk][currentItem]	=	storage[lastBunk][lastItem];
			}


private:
	T		**storage;
	int		lastBunk,lastItem;
	int		currentBunk,currentItem;
	int		maxBunk,bunkStep,itemsPerbunk;
};


///////////////////////////////////////////////////////////////////////
// Class				:	CPqueue
// Description			:	Priority queue
// Comments				:
// Date last edited		:	11/28/2001
template <class ItemType,class PriorityType> class CPqueue  {
	class	CPqueueItem	{
	public:
		ItemType		item;
		PriorityType	priority;
		int				*itemNumber;
	};
public:
					CPqueue()	{
						maxItems	=	100;
						numItems	=	1;
						allItems	=	new CPqueueItem[maxItems];
					}

		virtual		~CPqueue()	{
						delete [] allItems;
					}

		void		insert(ItemType item,PriorityType priority,int *itemNumber) {
						int i;

						if (numItems >= maxItems) {
							CPqueueItem					*newItems;
							maxItems					+=	100;
							newItems					=	new CPqueueItem[maxItems];
							for (i=0;i<numItems;i++) {
								newItems[i].item		=	allItems[i].item;
								newItems[i].priority	=	allItems[i].priority;
								newItems[i].itemNumber	=	allItems[i].itemNumber;
							}

							delete [] allItems;
							allItems	=	newItems;
						}
		
						i = numItems++;
						while ((i > 1) && (priority < allItems[i >> 1].priority)) {
							allItems[i].item			=	allItems[i >> 1].item;
							allItems[i].priority		=	allItems[i >> 1].priority;
							allItems[i].itemNumber		=	allItems[i >> 1].itemNumber;
							allItems[i].itemNumber[0]	=	i;
							i /= 2;
						}
						allItems[i].item			=	item;
						allItems[i].priority		=	priority;
						allItems[i].itemNumber		=	itemNumber;
						allItems[i].itemNumber[0]	=	i;
					}

		void		changePriority(int item,PriorityType newPriority) {
						int			i;
						ItemType	changingItem;
						int			*itemNumber;

						changingItem	=	allItems[item].item;
						itemNumber		=	allItems[item].itemNumber;
						
						i	=	item;
						if (newPriority < allItems[item].priority) {		// Need to push up
							while ((i > 1) && (newPriority < allItems[i >> 1].priority)) {
								allItems[i].item			=	allItems[i >> 1].item;
								allItems[i].priority		=	allItems[i >> 1].priority;
								allItems[i].itemNumber		=	allItems[i >> 1].itemNumber;

								allItems[i].itemNumber[0]	=	i;
								i /= 2;
							}
							allItems[i].item		=	changingItem;
							allItems[i].priority	=	newPriority;
							allItems[i].itemNumber	=	itemNumber;
							itemNumber[0]			=	i;
						} else {											// Need to push down
							int	j;
							while (i <= numItems / 2) {
								j = 2 * i;
								if (j >= numItems) break;

								if (j < (numItems-1) && 
									allItems[j].priority > allItems[j+1].priority) {
									j++;
								}
								if (allItems[j].priority > newPriority) {
									break;
								}

								allItems[i].priority		=	allItems[j].priority;
								allItems[i].item			=	allItems[j].item;
								allItems[i].itemNumber		=	allItems[j].itemNumber;
								allItems[i].itemNumber[0]	=	i;
								i = j;
							}
							allItems[i].priority	=	newPriority;
							allItems[i].item		=	changingItem;
							allItems[i].itemNumber	=	itemNumber;
							itemNumber[0]			=	i;
						}
					}


		int			getMax(ItemType *item,PriorityType *priority) {
						if (numItems <= 1) return 0;
						else {
							item[0]		=	allItems[1].item;
							priority[0]	=	allItems[1].priority;
							return 1;
						}
					}

		void		remove(int item) {
						int				i;
						PriorityType	oldPriority,newPriority;

						numItems--;

						oldPriority	=	allItems[item].priority;
						newPriority	=	allItems[numItems].priority;

						i	=	item;
						if (oldPriority > newPriority) {		// Need to push up
							while ((i > 1) && (newPriority < allItems[i >> 1].priority)) {
								allItems[i].item			=	allItems[i >> 1].item;
								allItems[i].priority		=	allItems[i >> 1].priority;
								allItems[i].itemNumber		=	allItems[i >> 1].itemNumber;
								allItems[i].itemNumber[0]	=	i;
								i /= 2;
							}
							allItems[i].item			=	allItems[numItems].item;
							allItems[i].priority		=	allItems[numItems].priority;
							allItems[i].itemNumber		=	allItems[numItems].itemNumber;
							allItems[i].itemNumber[0]	=	i;
						} else {											// Need to push down
							int	j;
							while (i <= numItems / 2) {
								j = 2 * i;
								if (j >= numItems) break;

								if (j < (numItems-1) && 
									allItems[j].priority > allItems[j+1].priority) {
									j++;
								}
								if (allItems[j].priority > newPriority) {
									break;
								}

								allItems[i].priority		=	allItems[j].priority;
								allItems[i].item			=	allItems[j].item;
								allItems[i].itemNumber		=	allItems[j].itemNumber;
								allItems[i].itemNumber[0]	=	i;
								i = j;
							}
							allItems[i].priority		=	allItems[numItems].priority;
							allItems[i].item			=	allItems[numItems].item;
							allItems[i].itemNumber		=	allItems[numItems].itemNumber;
							allItems[i].itemNumber[0]	=	i;
						}
					}

		int			removeMax(ItemType *item,PriorityType *priority) {
						int			i = 1, j;
						CPqueueItem	*lItem;

						if (numItems <= 1) return 0;
					
						item[0]		=	allItems[1].item;
						priority[0]	=	allItems[1].priority;
					
						numItems--;
						lItem	=	&allItems[numItems];

						while (i <= numItems / 2) {
							j = 2 * i;
							if (j >= numItems) break;

							if (j < (numItems-1) && 
								allItems[j].priority > allItems[j+1].priority) {
								j++;
							}
							if (allItems[j].priority > lItem->priority) {
								break;
							}

							allItems[i].priority		=	allItems[j].priority;
							allItems[i].item			=	allItems[j].item;
							allItems[i].itemNumber		=	allItems[j].itemNumber;
							allItems[i].itemNumber[0]	=	i;
							i = j;
						}
						allItems[i].priority			=	lItem->priority;
						allItems[i].item				=	lItem->item;
						allItems[i].itemNumber			=	lItem->itemNumber;
						allItems[i].itemNumber[0]		=	i;
						return TRUE;
					}

	int				sanityCheck() {
						int i;
						for (i=0;i<numItems /2;i++) {
							int	j	=	i*2;
							if (allItems[j].priority < allItems[i].priority) return FALSE;
							j++;
							if (j < numItems)
								if (allItems[j].priority < allItems[i].priority) return FALSE;
						}
						return TRUE;
					}
	CPqueueItem		*allItems;
	int				numItems,maxItems;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CPqueue2
// Description			:	Another priority queue but this one can dynamically change items in the buffer
// Comments				:
// Date last edited		:	11/28/2001
template <class ItemType,class PriorityType> class CPqueue2  {
	class	CPqueueItem	{
	public:
		ItemType		item;
		PriorityType	priority;
	};
public:
					CPqueue2()	{
						maxItems	=	100;
						numItems	=	1;
						allItems	=	new CPqueueItem[maxItems];
					}

		virtual		~CPqueue2()	{
						delete [] allItems;
					}

		void		insert(ItemType item,PriorityType priority) {
						int i;

						if (numItems >= maxItems) {
							CPqueueItem					*newItems;
							maxItems					+=	100;
							newItems					=	new CPqueueItem[maxItems];
							for (i=0;i<numItems;i++) {
								newItems[i].item		=	allItems[i].item;
								newItems[i].priority	=	allItems[i].priority;
							}

							delete [] allItems;
							allItems	=	newItems;
						}
		
						i = numItems++;
						while ((i > 1) && (priority < allItems[i >> 1].priority)) {
							allItems[i].item			=	allItems[i >> 1].item;
							allItems[i].priority		=	allItems[i >> 1].priority;
							i /= 2;
						}
						allItems[i].item			=	item;
						allItems[i].priority		=	priority;
					}

		
		int			getMax(ItemType *item,PriorityType *priority) {
						if (numItems <= 1) return 0;
						else {
							item[0]		=	allItems[1].item;
							priority[0]	=	allItems[1].priority;
							return 1;
						}
					}

		
		int			removeMax(ItemType *item,PriorityType *priority) {
						int			i = 1, j;
						CPqueueItem	*lItem;

						if (numItems <= 1) return 0;
					
						item[0]		=	allItems[1].item;
						priority[0]	=	allItems[1].priority;
					
						numItems--;
						lItem	=	&allItems[numItems];

						while (i <= numItems / 2) {
							j = 2 * i;
							if (j >= numItems) break;

							if (j < (numItems-1) && 
								allItems[j].priority > allItems[j+1].priority) {
								j++;
							}
							if (allItems[j].priority > lItem->priority) {
								break;
							}

							allItems[i].priority		=	allItems[j].priority;
							allItems[i].item			=	allItems[j].item;
							i = j;
						}
						allItems[i].priority			=	lItem->priority;
						allItems[i].item				=	lItem->item;
						return TRUE;
					}

	int				sanityCheck() {
						int i;
						for (i=0;i<numItems /2;i++) {
							int	j	=	i*2;
							if (allItems[j].priority < allItems[i].priority) return FALSE;
							j++;
							if (j < numItems)
								if (allItems[j].priority < allItems[i].priority) return FALSE;
						}
						return TRUE;
					}

	CPqueueItem		*allItems;
	int				numItems,maxItems;
};


///////////////////////////////////////////////////////////////////////
// Class				:	CMemPool
// Description			:	Memory pool for a particular class that's allocated frequently
// Comments				:
// Date last edited		:	11/28/2001
template <class T>	class CMemPool {
	typedef union TItemLinker {
		T				itemType;
		TItemLinker		*pointer;
	} TItemLinker;

public:
	
						CMemPool(int bankStep = 20,int itemsPerBank = 100) {

							this->bankStep		=	bankStep;
							this->itemsPerBank	=	itemsPerBank;
							
							mBanks		=	bankStep;
							nBanks		=	0;
							banks		=	new TItemLinker*[mBanks];

							freeItems	=	NULL;
						}

						~CMemPool() {
							int		i;

							for (i=0;i<nBanks;i++) {
								delete [] banks[i];
							}

							delete [] banks;
						}

						T		*newItem() {
							if (freeItems != NULL) {
								T			*cItem;
								cItem		=	&freeItems->itemType;
								freeItems	=	freeItems->pointer;
								return cItem;	
							} else {
								int			i;

								if (nBanks == (mBanks-1)) {
									TItemLinker	**newItems;

									mBanks		+=	bankStep;

									newItems	=	new TItemLinker*[mBanks];

									for (i=0;i<nBanks;i++)
										newItems[i]	=	banks[i];

									delete [] banks;
									banks	=	newItems;
								}

								banks[nBanks]	=	new TItemLinker[itemsPerBank];

								for (i=0;i<itemsPerBank;i++) {
									banks[nBanks][i].pointer	=	freeItems;
									freeItems					=	&banks[nBanks][i];
								}

								nBanks++;

								return newItem();
							}
						}


						void	deleteItem(T *item) {
							((TItemLinker *) item)->pointer	=	freeItems;
							freeItems						=	(TItemLinker *) item;
						}


	TItemLinker			**banks;
	int					nBanks,mBanks;
	TItemLinker			*freeItems;
	int					bankStep,itemsPerBank;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CMemStack
// Description			:	A stack based memory manager
// Comments				:
// Date last edited		:
class	CMemStack {
	typedef	struct TBucketInfo {
		TBucketInfo		*next;			// Points to the next bucket
		TBucketInfo		*prev;			// Points to the previous bucket
		unsigned char	*memory;		// Points to the first block of available memory
	} TBucketInfo;
public:
						CMemStack(int bs = 100000) {
							bucketSize		=	bs;

							// Align the bucket size to 32 byte boundary
							memory			=	new unsigned char[bucketSize];
							available		=	bucketSize;
							bucketSize		=	(bucketSize & ~15) + 16;

							lastCheck		=	0;

							savedBuckets	=	NULL;
						}

						~CMemStack() {
							TBucketInfo	*cInfo,*nInfo;

							for (cInfo=savedBuckets;cInfo!=NULL;) {
								nInfo	=	cInfo->next;
								delete [] cInfo->memory;
								delete cInfo;
								cInfo	=	nInfo;
							}

							if (memory != NULL) {
								delete [] (unsigned char *) memory;
								memory	=	NULL;
							}
						}


		void			check() {
							// Save the current bucket/available bytes
							checkMemories[lastCheck]		=	memory;
							checkAvailables[lastCheck]	=	available;
							lastCheck++;
						}

		void			restore() {
							// Restore the memory status
							lastCheck--;
							if (checkMemories[lastCheck] != memory) {
								TBucketInfo	*cInfo,*nInfo;

								delete [] memory;

								for (cInfo=savedBuckets;cInfo!=NULL;) {
									nInfo	=	cInfo->next;
									if (cInfo->memory == checkMemories[lastCheck]) {
										memory			=	cInfo->memory;
										delete cInfo;
										savedBuckets	=	nInfo;
										if (savedBuckets != NULL) savedBuckets->prev		=	NULL;
										break;
									} else {
										delete [] cInfo->memory;
										delete cInfo;
									}
									cInfo	=	nInfo;
								}
							}

							available	=	checkAvailables[lastCheck];
						}

		void			*alloc(int size) {
							void	*data;

							// Align data to 16 byte boundary
							if (size & 15)	size	=	(size & ~15) + 16;

							if (available < size) {
								if (size < bucketSize) {
									TBucketInfo		*newInfo	=	new TBucketInfo;

									newInfo->next			=	savedBuckets;
									if (savedBuckets != NULL)
										savedBuckets->prev	=	newInfo;
									savedBuckets			=	newInfo;

									newInfo->memory			=	memory;

									memory					=	new unsigned char[bucketSize];
									available				=	bucketSize;
								} else {
									return NULL;
								}
							}

							data	=	&memory[bucketSize-available];

							available				-=	size;

							return	data;
						}

		TBucketInfo		*savedBuckets;

		int				lastCheck;
		unsigned char	*checkMemories[256];
		int				checkAvailables[256];

		unsigned char	*memory;
		int				available;
		int				bucketSize;

};

///////////////////////////////////////////////////////////////////////
// Class				:	CArray
// Description			:	Array
// Comments				:
// Date last edited		:	11/28/2001
template <class T> class CArray {
	public:
						CArray(int ss = 100) {
							arraySize	=	0;
							stepSize	=	ss;
							array		=	NULL;
							lastItem	=	0;
						}

						~CArray() {
							if (array != NULL)
								delete [] array;
						}


		T				& operator[](int index) {
							if (index >= arraySize) {
								if (arraySize == 0) {
									arraySize		=	stepSize;
									array			=	new T[arraySize];
								} else {
									T	*newArray	=	new T[arraySize+stepSize];

									mcpy(newArray,array,arraySize*sizeof(T));

									arraySize	+=	stepSize;
									delete [] array;
									array	=	newArray;
								}
							}

							return array[index];
						}

		T				& operator += (T item) {
							return (*this)[lastItem++]	=	item;
						}

		T				*getArray()	{
							return array;
						}

		int				getSize() {
							return lastItem;
						}

		void			setSize(int in) {
							lastItem	=	in;
						}

		void			reset() {
							lastItem	=	0;
						}

	private:
		T				*array;
		int				arraySize;
		int				stepSize;
		int				lastItem;
};



#endif


