// directX.cpp: implementation of the directX class.
//
//////////////////////////////////////////////////////////////////////

#include "directX.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
void InterpretError(HRESULT hr)
{
    char errStr[256];

    if(D3DXERR_CAPSNOTSUPPORTED == hr) 
    {
        strcpy(errStr, "D3DXERR_CAPSNOTSUPPORTED\n\n"
                       "This device lacks required capabilities.  "
                       "Try using the reference rasterizer.");
    }
    else
    {
        D3DXGetErrorString(hr, 256, errStr);
    }

    MessageBox(NULL, errStr, "D3DX Error", MB_OK);
}

CDirectX::CDirectX(HWND hwnd,CGraphics *desc) {
	HRESULT	hr;
	DWORD	flags;
	DWORD	buffers;


	directContext	=	NULL;
	directDraw		=	NULL;
	direct3D		=	NULL;
	direct3DDevice	=	NULL;

	hr = D3DXInitialize();

	if (FAILED(hr)) return;

	flags	=	0;
	if (desc->fullScreen == TRUE)
		flags	=	D3DX_CONTEXT_FULLSCREEN;
	if (desc->offScreen == TRUE)
		flags	=	D3DX_CONTEXT_OFFSCREEN;

	if (desc->doubleBuffered == TRUE)
		buffers	=	1;
	else
		buffers	=	0;

	hr = D3DXCreateContextEx(D3DX_DEFAULT,		// Device
							flags,				// Flags (Full screen ?)
							hwnd,				// Window
							NULL,				// Focus Window
							D3DX_DEFAULT,	// Color Bits
							0,	// Alpha Bits
							D3DX_DEFAULT,	// Depth Bits
							0,	// Stencil Bits
							D3DX_DEFAULT,			// Back buffers
							D3DX_DEFAULT,		// Width
							D3DX_DEFAULT,		// Height
							D3DX_DEFAULT,		// Refresh Rate
							&directContext);	// Context

	if (FAILED(hr)) {
		InterpretError(hr);
		return;
	}

	if(!(directDraw = directContext->GetDD()))
        return;

    if(!(direct3D = directContext->GetD3D()))
        return;

    if(!(direct3DDevice = directContext->GetD3DDevice()))
        return;
}

CDirectX::~CDirectX() {
	if (directContext != NULL) {
		direct3DDevice->Release();
		direct3D->Release();
		directDraw->Release();
		directContext->Release();
	}
}
