// openGL.cpp: implementation of the openGL class.
//
//////////////////////////////////////////////////////////////////////

#include "openGL.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int ChoosePixelFormatEx(HDC hdc,int *p_bpp,int *p_depth,int *p_dbl,int *p_acc)
{ int wbpp; if (p_bpp==NULL) wbpp=-1; else wbpp=*p_bpp;
  int wdepth; if (p_depth==NULL) wdepth=16; else wdepth=*p_depth;
  int wdbl; if (p_dbl==NULL) wdbl=-1; else wdbl=*p_dbl;
  int wacc; if (p_acc==NULL) wacc=1; else wacc=*p_acc;
  PIXELFORMATDESCRIPTOR pfd; ZeroMemory(&pfd,sizeof(pfd)); pfd.nSize=sizeof(pfd); pfd.nVersion=1;
  int num=DescribePixelFormat(hdc,1,sizeof(pfd),&pfd);
  if (num==0) return 0;
  unsigned int maxqual=0; int maxindex=0;
  int max_bpp, max_depth, max_dbl, max_acc;
  for (int i=1; i<=num; i++)
  { ZeroMemory(&pfd,sizeof(pfd)); pfd.nSize=sizeof(pfd); pfd.nVersion=1;
    DescribePixelFormat(hdc,i,sizeof(pfd),&pfd);
    int bpp=pfd.cColorBits;
    int depth=pfd.cDepthBits;
    bool pal=(pfd.iPixelType==PFD_TYPE_COLORINDEX);
    bool mcd=((pfd.dwFlags & PFD_GENERIC_FORMAT) && (pfd.dwFlags & PFD_GENERIC_ACCELERATED));
    bool soft=((pfd.dwFlags & PFD_GENERIC_FORMAT) && !(pfd.dwFlags & PFD_GENERIC_ACCELERATED));
    bool icd=(!(pfd.dwFlags & PFD_GENERIC_FORMAT) && !(pfd.dwFlags & PFD_GENERIC_ACCELERATED));
    unsigned int opengl= (unsigned int) (pfd.dwFlags & PFD_SUPPORT_OPENGL);
    unsigned int window= (unsigned int) (pfd.dwFlags & PFD_DRAW_TO_WINDOW);
    unsigned int bitmap= (unsigned int) (pfd.dwFlags & PFD_DRAW_TO_BITMAP);
    bool dbuff=(pfd.dwFlags & PFD_DOUBLEBUFFER);
    //
    unsigned int q=0;
    if (opengl && window) q=q+0x8000;
    if (wdepth==-1 || (wdepth>0 && depth>0)) q=q+0x4000;
    if (wdbl==-1 || (wdbl==0 && !dbuff) || (wdbl==1 && dbuff)) q=q+0x2000;
    if (wacc==-1 || (wacc==0 && soft) || (wacc==1 && (mcd || icd))) q=q+0x1000;
    if (mcd || icd) q=q+0x0040; if (icd) q=q+0x0002;
    if (wbpp==-1 || (wbpp==bpp)) q=q+0x0800;
    if (bpp>=16) q=q+0x0020; if (bpp==16) q=q+0x0008;
    if (wdepth==-1 || (wdepth==depth)) q=q+0x0400;
    if (depth>=16) q=q+0x0010; if (depth==16) q=q+0x0004;
    if (!pal) q=q+0x0080;
    if (bitmap) q=q+0x0001;
    if (q>maxqual) {maxqual=q; maxindex=i;max_bpp=bpp; max_depth=depth; max_dbl=dbuff?1:0; max_acc=soft?0:1;}
  }
  if (maxindex==0) return maxindex;
  if (p_bpp!=NULL) *p_bpp=max_bpp;
  if (p_depth!=NULL) *p_depth=max_depth;
  if (p_dbl!=NULL) *p_dbl=max_dbl;
  if (p_acc!=NULL) *p_acc=max_acc;
  return maxindex;
}


COpenGL::COpenGL(HWND	hwnd,CGraphics *desc) {
	// Get the device context
	glWindow	=	hwnd;
	glDC		=	GetDC(glWindow);

	
	int			bpp		=	desc->bpp;				// don't care. (or a positive integer)
	int			depth	=	desc->depthBits;		// don't care. (or a positive integer)
	int			dbl		=	desc->doubleBuffered;	// we want double-buffering. (or -1 for 'don't care', or 0 for 'none')
	int			acc		=	TRUE;					// we want acceleration. (or -1 or 0)

	if (desc->fullScreen) {
		DWORD		dwExStyle;						// Window Extended Style
		DWORD		dwStyle;
		RECT		WindowRect;
		DEVMODE		dmScreenSettings;

		WindowRect.left		=	(long)0;
		WindowRect.right	=	(long)desc->width;
		WindowRect.top		=	(long)0;
		WindowRect.bottom	=	(long)desc->height;		

		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));
		dmScreenSettings.dmSize			= sizeof(dmScreenSettings);
		dmScreenSettings.dmPelsWidth	= desc->width;
		dmScreenSettings.dmPelsHeight	= desc->height;
		dmScreenSettings.dmBitsPerPel	= 32;
		dmScreenSettings.dmFields		= DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL) {
			dwExStyle	=	WS_EX_APPWINDOW;
			dwStyle		=	WS_POPUP;
			AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);
			fullScreen	=	TRUE;
			width		=	desc->width;
			height		=	desc->height;
		} else {
			ChangeDisplaySettings(NULL,0);
			fullScreen	=	FALSE;
		}
	}

	int pf=ChoosePixelFormatEx(glDC,&bpp,&depth,&dbl,&acc);

	this->bpp				=	bpp;
	this->depthBits			=	depth;
	this->doubleBuffered	=	dbl;

	PIXELFORMATDESCRIPTOR pfd; ZeroMemory(&pfd,sizeof(pfd)); pfd.nSize=sizeof(pfd); pfd.nVersion=1;

	if (SetPixelFormat(glDC,pf,&pfd) == FALSE) 
		glContext	=	NULL;
	else
		glContext	=	wglCreateContext(glDC);	
}

COpenGL::~COpenGL() {
	if (fullScreen) {
		// Switch back to the desktop
		ChangeDisplaySettings(NULL,0);
	}

	wglMakeCurrent(glDC,NULL);
	wglDeleteContext(glContext);
	ReleaseDC(glWindow,glDC);
}

char	*COpenGL::getDescription() {
	return NULL;
}

