///////////////////////////////////////////////////////////////////////
//
//  File				:	os.cpp
//  Classes				:	COs
//  Description			:
//
//
//
//	Ver 11/28/2001
//
////////////////////////////////////////////////////////////////////////


#include "os.h"
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

// Some operating system specific stuff
#ifdef WIN32
#define	OS_DIR_SEPERATOR					'\\'
#define OS_DIR_SEPERATOR_STRING				"\\"
#else
#define OS_DIR_SEPERATOR					'/'
#define OS_DIR_SEPERATOR_STRING				"/"
#endif

#define OS_ENVIRONMENT_SEPERATOR			':'
#define OS_ENVIRONMENT_SEPERATOR_STRING		":"

#define	OS_ENVIRONMENT_MARKER				'$'
#define	OS_ENVIRONMENT_MARKER_STRING		"$"

#ifdef WIN32
char	*COs::moduleExtension							=	"dll";
char	*COs::executableExtension						=	"exe";
#else
char	*COs::moduleExtension							=	"so";
char	*COs::executableExtension						=	"";
#endif

///////////////////////////////////////////////////////////////////////
// Function				:	isAbsolute
// Description			:	Is the fileName absolute ?
// Return Value			:	TRUE if it is
// Comments				:
// Date last edited		:	11/28/2001
static	int		isAbsolute(const char *fileName) {
#ifdef WIN32
	if (fileName[1] == ':')	return TRUE;
	if (fileName[0] == '\\')	return TRUE;
#else
	if (fileName[0] == '\\')	return TRUE;
#endif
	return	FALSE;
}

/////////////////////////////////////////////////////////////////////////////////////
//
//
//	The implementation of the COS class
//
//	This class must handle all the OS specific stuff
//
//	Note that this class is entirely static so it requires no instance




///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	COs
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
/////////////////////////////////////////////////////////////////////////////////////
COs::COs(){
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	~COs
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
COs::~COs(){
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	loadModule
// Description			:	Load / attach a dll/so file
// Return Value			:	The module handle
// Comments				:
// Date last edited		:	11/28/2001
void	*COs::loadModule(char *name) {
	void	*cModule	=	NULL;

	if (name != NULL) {
#ifdef WIN32
	// Win32 stuff here
	cModule	=	LoadLibrary(name);

#else
#ifdef __DJGPP__
	if (!LoadDLM(name)) {
		// Unable to open the file
	} else {
		cModule	=	sdup(name);
	}
#else
	// Unix stuff here
	cModule = dlopen(name,RTLD_NOW);
#endif

#endif
	}

	return cModule;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	unloadModule
// Description			:	Unload / detach a dll/so file
// Return Value			:	The module handle
// Comments				:
// Date last edited		:	11/28/2001
void	COs::unloadModule(void *cModule) {
#ifdef WIN32
	if (cModule != NULL) FreeLibrary((HMODULE) cModule);
#else
#ifdef __DJGPP__
	sfree((char *) cModule);
#else
	if (cModule != NULL) dlclose((void *) cModule);
#endif
#endif
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	resolve
// Description			:	Resolve a symbol in a dll/so file
// Return Value			:	The module handle
// Comments				:
// Date last edited		:	11/28/2001
void	*COs::resolve(void *cModule,char *name) {
	void	*result	=	NULL;
#ifdef WIN32
	if (cModule != NULL) {
		result	=	GetProcAddress((HMODULE) cModule,name);
	}
#else
#ifdef __DJGPP__
	if (cModule != NULL) {
		if (!LoadDLM((char *) cModule))	return	NULL;

		return	LookupDLMSymbol(name);
	}
#else
	if (cModule != NULL) {
		result	=	dlsym((void *) cModule,name);
	}
#endif
#endif

	return result;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	environment
// Description			:	Return an environment variable
// Return Value			:	The str
// Comments				:
// Date last edited		:	11/28/2001
char	*COs::environment(const char *name) {
	if (name == NULL) return NULL;

	return getenv(name);
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	memalloc
// Description			:	Allocate memory
// Return Value			:	Memory
// Comments				:
// Date last edited		:	11/28/2001
void	*COs::memmalloc(const int size) {
	void *ptr	=	malloc(size);
	if (ptr == NULL) {
		// Not enough memory ... need to make some
		exit();
	}
	return ptr;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	memfree
// Description			:	Free memory
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
void	COs::memfree(void *ptr) {
	free(ptr);
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	exit
// Description			:	Exit
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
void	COs::exit() {
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	fileExists
// Description			:	Check if a file exists
// Return Value			:	TRUE if it does
// Comments				:
// Date last edited		:	11/28/2001
int		COs::fileExists(const char *name) {
#ifdef WIN32
	if (_access(name,0) == 0) return TRUE;
#else
	if (access(name,0) == 0) return TRUE;
#endif
	return FALSE;
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	locateFile
// Description			:	Locate a file on disk
// Return Value			:	The location of the file
// Comments				:
// Date last edited		:	11/28/2001
int		COs::locateFile(char *result,const char *fileName,const char *prefix) {
	char			fullPath[OS_MAX_PATH_LENGTH];
		
	getAbsolutePath(fullPath,fileName);

	if (fileExists(fullPath)) {
		scpy(result,fullPath);
		return	TRUE;
	}

	getFileName(fullPath,fileName);

	if (prefix != NULL) {
		char			tmp[OS_MAX_PATH_LENGTH];
		char			filePath[OS_MAX_PATH_LENGTH];
		const	char	*currentPath;

		currentPath	=	prefix;

		while(currentPath[0]	!=	'\0') {
			char	*endOfCurrentPath	=	schr(currentPath,OS_ENVIRONMENT_SEPERATOR);

			// Copy the prefix into out
			if (endOfCurrentPath != NULL) {
				scpy(tmp,currentPath,endOfCurrentPath-currentPath);
				tmp[endOfCurrentPath-currentPath]	=	'\0';
			} else
				scpy(tmp,currentPath);

			// Figure out if we need to substitude any environment variables
			getFileName(filePath,fullPath,tmp);
			getAbsolutePath(tmp,filePath);
			
			// Check the existance of the file
			if (fileExists(tmp)) {
				scpy(result,tmp);
				return TRUE;
			}

			if (endOfCurrentPath == NULL) break;
			else currentPath	=	(endOfCurrentPath+1);
		}
	}

	return FALSE;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	getFileName
// Description			:	Concetenate the filename with a prefix
// Return Value			:	The location of the file
// Comments				:
// Date last edited		:	11/28/2001
void	COs::getFileName(char *result,const char *fileName,const char *prefix) {
	char	tmp[OS_MAX_PATH_LENGTH];

	if (prefix == NULL) {
		char	*seperator;

		scpy(tmp,fileName);
		fixSlashes(tmp);

		seperator			=	srchr(tmp,OS_DIR_SEPERATOR);

		if (seperator == NULL) {
			scpy(result,tmp);
		} else {
			scpy(result,seperator+1);
		}
	} else {
		char	fn[OS_MAX_PATH_LENGTH];

		getFileName(fn,fileName);

		scpy(tmp,prefix);

		while(TRUE) {
			char	*leftMarker;
			leftMarker	=	schr(tmp,OS_ENVIRONMENT_MARKER);
			if (leftMarker != NULL) {
				char	*rightMarker	=	schr(leftMarker+1,OS_ENVIRONMENT_MARKER);
				if (rightMarker != NULL) {
					char	tmp2[OS_MAX_PATH_LENGTH];
					char	*env;

					// Extract the variable name
					scpy(tmp2,leftMarker+1,rightMarker-leftMarker-1);
					tmp2[rightMarker-leftMarker-1]	=	'\0';

					// Get the variable
					env	=	environment(tmp2);

					// Copy the postix into tmp
					scpy(tmp2,rightMarker+1);

					if (env != NULL) {
						sprintf(leftMarker,"%s",env);
					} else {
						sprintf(leftMarker,"");
					}

					scat(leftMarker,tmp2);
				} else break;
			} else break;
		}

		if (tmp[slen(tmp)-1] != OS_DIR_SEPERATOR)
			scat(tmp,OS_DIR_SEPERATOR_STRING);
		scat(tmp,fn);

		getAbsolutePath(result,tmp);
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	getAbsolutePath
// Description			:	Get the absolute location of a file
// Return Value			:	The location of the file
// Comments				:
// Date last edited		:	11/28/2001
void	COs::getAbsolutePath(char *result,const char *path) {
	if (isAbsolute(path)) {
		scpy(result,path);
	} else {
#ifdef WIN32
		_fullpath(result,path,OS_MAX_PATH_LENGTH);
#else
		{
			char	realp[PATH_MAX];
			if (realpath(path,realp) != NULL) {
				scpy(result,realp);
			} else {
				scpy(result,path);
			}
		}
#endif

	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	fixSlashes
// Description			:	Make sure the slashes are right
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
void	COs::fixSlashes(char *st) {
	int	i;

	for (i=0;i<st[i] != '\0';i++) {
		if ((st[i] == '\\') || (st[i] == '/')) st[i]	=	OS_DIR_SEPERATOR;
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	pipeopen
// Description			:	Create a pipe
// Return Value			:	FILE
// Comments				:
// Date last edited		:	11/28/2001
FILE	*COs::pipeopen(const char *name,const char *mode) {
#ifdef WIN32
	return _popen(name,mode);
#else
	return	popen(name,mode);
#endif
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	pipeclose
// Description			:	Close the pipe
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
void	COs::pipeclose(FILE *h) {
#ifdef WIN32
	_pclose(h);
#else
	pclose(h);
#endif
}



///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	mkdir
// Description			:	Create a dir
// Return Value			:	-
// Comments				:
// Date last edited		:	11/28/2001
void	COs::mkdir(const char *name) {
	char	tmp[128];

	sprintf(tmp,"mkdir %s",name);
	fixSlashes(tmp);
	system(tmp);
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	rmdir
// Description			:	Delete a dir
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
void	COs::rmdir(const char *name) {
	char	tmp[128];

	sprintf(tmp,"rmdir %s",name);
	fixSlashes(tmp);
	system(tmp);
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	time
// Description			:	Get the time
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
float	COs::time() {
#ifdef WIN32
#else
#define	_timeb	timeb
#define	_ftime	ftime
#endif

	_timeb timebuffer;
	
	_ftime( &timebuffer );

	return	(float) (timebuffer.time + timebuffer.millitm / (float) 1000);
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	cpuTime
// Description			:	Get the cpu time
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
float	COs::cpuTime() {
	return	clock() / (float) CLOCKS_PER_SEC;
}


///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	createThread
// Description			:	Start a thread
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
TThread	COs::createThread(TFun entry,void *data) {
	TThread	cThread;

#ifdef WIN32
	DWORD	id;

	cThread	=	CreateThread(NULL,0,entry,data,0,&id);
#else
	pthread_create(&cThread,NULL,entry,data);
#endif

	return	cThread;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	waitThread
// Description			:	Wait for the termination of a thread
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
int		COs::waitThread(TThread	thread) {
#ifdef WIN32
	WaitForSingleObject(thread,INFINITE);
#else
	return pthread_join(thread,NULL);
#endif

	return 0;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	createMutex
// Description			:	Create a mutex
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
TMutex	COs::createMutex() {
	TMutex	mutex;
#ifdef WIN32
	mutex	=	CreateMutex(NULL,FALSE,NULL);
#else
	pthread_mutex_init(&mutex,NULL);
#endif

	return mutex;
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	downMutex
// Description			:	Lock a mutex
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
void	COs::downMutex(TMutex mutex) {
#ifdef WIN32
	WaitForSingleObject(mutex,INFINITE);
#else
	pthread_mutex_lock(&mutex);
#endif
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	upMutex
// Description			:	Unlock a mutex
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
void	COs::upMutex(TMutex mutex) {
#ifdef WIN32
	ReleaseMutex(mutex);
#else
	pthread_mutex_unlock(&mutex);
#endif
}

///////////////////////////////////////////////////////////////////////
// Class				:	COs
// Method				:	deleteMutex
// Description			:	Delete a mutex
// Return Value			:
// Comments				:
// Date last edited		:	11/28/2001
void	COs::deleteMutex(TMutex mutex) {
#ifdef WIN32
	CloseHandle(mutex);
#else
	pthread_mutex_destroy(&mutex);
#endif
}

