///////////////////////////////////////////////////////////////////////
//
//  File				:	os.h
//  Classes				:	COs
//  Description			:	OS dependent routines
//
//
//
//	Ver 11/28/2001
//
////////////////////////////////////////////////////////////////////////
#ifndef OS_H
#define OS_H

/* This class encapsulates all disk/network IO	*/
#include "global.h"
#include "containers.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

/* All system dependent headers come here			*/
#ifdef WIN32
// Windoze specific headers
#include <io.h>
#include <windows.h>

#define	LIB_EXPORT				__declspec(dllexport)
#define LIB_IMPORT				__declspec(dllimport)

#else
// Unix specific headers
#ifdef __DJGPP__
#include <dlmlib.h>
#include <unistd.h>
#include <limits.h>
#else
#include <dlfcn.h>
#include <unistd.h>
#include <pthread.h>
#include <limits.h>
#endif


#define	LIB_EXPORT				extern
#define LIB_IMPORT				extern

#endif

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


// MAximum length of a path
const	int				OS_MAX_PATH_LENGTH	=	512;
const	int				OS_NUM_PATHS		=	4;

#ifdef WIN32
#define	TThreadSpec		DWORD WINAPI
#define	TThread			HANDLE
#define	TMutex			HANDLE
#define	TFun			LPTHREAD_START_ROUTINE 
#else
#define	TThreadSpec		void *
#define	TThread			pthread_t
#define	TMutex			pthread_mutex_t
typedef void			*(*TFun)(void *);
#endif

// Generic io class. All the library io is done thru this class
class COs {
public: 
						COs();
						~COs();

	// Module I/O
	static	void		*loadModule(char *);
	static	void		unloadModule(void *);
	static	void		*resolve(void *,char *);

	// Get an environment variable (return NULL if non existent)
	static	char		*environment(const char *);
	
	/* Memory management			*/
	static	void		*memmalloc(const int);
	static	void		memfree(void *);

	/* Program termination			*/
	static	void		exit();

	/* File I/O stuff				*/
	static	int			fileExists(const char *);
	static	int			locateFile(char *,const char *,const char *prefix = NULL);
	static	void		getFileName(char *,const char *,const char *prefix = NULL);
	static	void		getAbsolutePath(char *,const char *);
	static	void		fixSlashes(char *);

	/* Additional file I/O			*/
	static	void		mkdir(const char *);
	static	void		rmdir(const char *);

	/* Misc I/O						*/
	static	FILE		*pipeopen(const char *,const char *);
	static	void		pipeclose(FILE *);

	/* Time functions				*/
	static	float		time();
	static	float		cpuTime();

	/* Thread mutex junk */
	static	TThread		createThread(TFun,void *);
	static	int			waitThread(TThread);
	static	TMutex		createMutex();
	static	void		deleteMutex(TMutex);
	static	void		downMutex(TMutex);
	static	void		upMutex(TMutex);

	/* Standard extensions */
	static	char		*moduleExtension;
	static	char		*executableExtension;
private:
};

#endif

