///////////////////////////////////////////////////////////////////////
//
//  File				:	simulation.cpp
//  Classes				:	CGeometry
//							CForce
//							CRigidBody
//							CCollisionDetector
//							CSimulation
//  Description			:	Implementation
//
//
//
//
//
////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "Simulation.h"

#include "mySimulation.h"

///////////////////////////////////////////////////////////////////////
// Class				:	CGeometry
// Method				:	CGeometry
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CGeometry::CGeometry() {
}

///////////////////////////////////////////////////////////////////////
// Class				:	CGeometry
// Method				:	~CGeometry
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CGeometry::~CGeometry() {
}

///////////////////////////////////////////////////////////////////////
// Class				:	CGeometry::CTriangle
// Method				:	computeNorm
// Description			:	Compute the normal vector of the triangle
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CGeometry::CTriangle::computeNorm(vector *vertices) {
	vector	v1,v2;

	subvv(v1,vertices[v[2]],vertices[v[1]]);
	subvv(v2,vertices[v[0]],vertices[v[1]]);
	crossvv(norm,v1,v2);
	normalizev(norm);
	d	=	-dotvv(norm,vertices[v[0]]);
}

///////////////////////////////////////////////////////////////////////
// Class				:	CGeometry::CTriangle
// Method				:	finalize
// Description			:	Compute the normal vector of the triangle
// Return Value			:	-
// Comments				:
// Date last edited		:	4/17/2001
void	CGeometry::finalize() {
	int	i;

	for (i=0;i<numTriangles;i++) {
		triangles[i].computeNorm(vertices);
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CRigidBody
// Method				:	CRigidBody
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CRigidBody::CRigidBody(CCollisionDetector *cd,CArray<CGeometry *> *g,float mass) {
	int						i;
	matrix					mtmp;

	geometry			=	g;

	for (i=0;i<g->getSize();i++)
		(*geometry)[i]->finalize();

	currentState			=	&states[0];
	lastStableState			=	&states[1];

	computeInertiaTensor(COM,inertiaTensor,mass,geometry);
	invertm(invInertiaTensor,inertiaTensor);

	{
		int j;

		for (j=0;j<g->getSize();j++) {
			for (i=0;i<(*geometry)[j]->numVertices;i++) {
				subvv((*geometry)[j]->vertices[i],COM);
			}

			(*geometry)[j]->finalize();
		}
	}

	lastStableState->clock	=	0;
	initv(lastStableState->avelocity,0,0,0);	// No angular velocity
	initv(lastStableState->velocity,0,0,0);		// No linear velocity
	initv(lastStableState->P,0,0,0);			// No linear momentum
	initv(lastStableState->L,0,0,0);			// No angular momentum

	movvv(lastStableState->position,COM);		// Position is set to COM
	lastStableState->orientation[0]	=	1;		// Identity orientation
	lastStableState->orientation[1]	=	0;
	lastStableState->orientation[2]	=	0;
	lastStableState->orientation[3]	=	0;

	flags						=	0;	

	this->mass					=	mass;
	if (mass == 0) {
		int	i;
		invMass					=	0;
		for (i=0;i<16;i++) {
			inertiaTensor[i]	=	0;
			invInertiaTensor[i]	=	0;
		}

		inertiaTensor[15]		=	1;
		invInertiaTensor[15]	=	1;
		flags					|=	RBODY_FIXED;
	} else
		invMass					=	1/mass;

												// Compute the auxilary matrices
	rotatem(lastStableState->R,lastStableState->orientation);
	transposem(lastStableState->Rt,lastStableState->R);
	mulmm(mtmp,invInertiaTensor,lastStableState->Rt);
	mulmm(lastStableState->Iinv,lastStableState->R,mtmp);

	

	collisionHandle				=	cd->registerGeometry(this,geometry,(mass == 0 ? TRUE : FALSE));
	cd->setState(collisionHandle,lastStableState->R,lastStableState->position);
}

///////////////////////////////////////////////////////////////////////
// Class				:	CRigidBody
// Method				:	~CRigidBody
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CRigidBody::~CRigidBody() {
	if (geometry != NULL)
		delete geometry;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CRigidBody
// Method				:	advance
// Description			:	Integrates the equations of motion for 'dTime'
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CRigidBody::advance(float dTime) {
	vector	tmp,tmp2;
	matrix	mtmp;

	// If this flag is set, we don't move
	if (flags & RBODY_FIXED) {
		mcpy(currentState,lastStableState,sizeof(CState));
		currentState->clock	=	lastStableState->clock+dTime;
		return;
	}

	// The linear momentum & velocity
	mulvf(tmp,netForce,dTime);
	addvv(currentState->P,lastStableState->P,tmp);
	
	// The position
	mulvf(currentState->position,lastStableState->velocity,dTime);
	addvv(currentState->position,lastStableState->position);

	// The angular momentum
	mulvf(tmp,netTorque,dTime);
	addvv(currentState->L,lastStableState->L,tmp);
	
	// The orientation
	currentState->orientation[0]	=	-dotvv(lastStableState->avelocity,&lastStableState->orientation[1]);
	
	crossvv(tmp2,lastStableState->avelocity,&lastStableState->orientation[1]);
	mulvf(&currentState->orientation[1],lastStableState->avelocity,lastStableState->orientation[0]);
	addvv(&currentState->orientation[1],tmp2);

	currentState->orientation[0]	*=	(float) 0.5*dTime;
	currentState->orientation[1]	*=	(float) 0.5*dTime;
	currentState->orientation[2]	*=	(float) 0.5*dTime;
	currentState->orientation[3]	*=	(float) 0.5*dTime;

	currentState->orientation[0]	+=	lastStableState->orientation[0];
	currentState->orientation[1]	+=	lastStableState->orientation[1];
	currentState->orientation[2]	+=	lastStableState->orientation[2];
	currentState->orientation[3]	+=	lastStableState->orientation[3];

	{	// Re - normalize the orientation
		float	tmp;

		tmp	=	currentState->orientation[0] * currentState->orientation[0]	+
				currentState->orientation[1] * currentState->orientation[1]	+
				currentState->orientation[2] * currentState->orientation[2]	+
				currentState->orientation[3] * currentState->orientation[3];

		tmp	=	(float) sqrt(tmp);

		currentState->orientation[0]	/=	tmp;
		currentState->orientation[1]	/=	tmp;
		currentState->orientation[2]	/=	tmp;
		currentState->orientation[3]	/=	tmp;
	}

	// Compute the auxilary variables
	rotatem(currentState->R,currentState->orientation);
	transposem(currentState->Rt,currentState->R);
	mulmm(mtmp,invInertiaTensor,currentState->Rt);
	mulmm(currentState->Iinv,currentState->R,mtmp);

	mulmv(currentState->avelocity,currentState->Iinv,currentState->L);

	mulvf(currentState->velocity,lastStableState->P,invMass);
		
	// Update the clock
	currentState->clock	=	lastStableState->clock + dTime;
}


///////////////////////////////////////////////////////////////////////
// Class				:	CRigidBody
// Method				:	rollback
// Description			:	Rollback the state to the last stable one
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CRigidBody::rollback() {
	// No action needed
}

///////////////////////////////////////////////////////////////////////
// Class				:	CRigidBody
// Method				:	commit
// Description			:	Make the current state stable
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CRigidBody::commit() {
	CState	*tmp	=	lastStableState;
	lastStableState	=	currentState;
	currentState	=	tmp;
}


///////////////////////////////////////////////////////////////////////
// Class				:	CCollisionDetector
// Method				:	CCollisionDetector
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CCollisionDetector::CCollisionDetector() {
	tolerance	=	(float) 0.001;
	flags		=	0;

	mContacts	=	100;
	nContacts	=	0;
	contacts	=	new CContact[mContacts];

	coefficientsOfFriction		=	NULL;
	coefficientsOfRestitution	=	NULL;
	numBodies					=	0;
	maxBodies					=	0;

	defaultFriction				=	0.5f;
	defaultRestitution			=	0.5f;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CCollisionDetector
// Method				:	~CCollisionDetector
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CCollisionDetector::~CCollisionDetector() {
	delete [] contacts;

	if (coefficientsOfFriction != NULL) {
		delete [] coefficientsOfFriction[0];
		delete [] coefficientsOfFriction;
	}

	if (coefficientsOfRestitution != NULL) {
		delete [] coefficientsOfRestitution[0];
		delete [] coefficientsOfRestitution;
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CCollisionDetector
// Method				:	registerGeometry
// Description			:	Add a geometry into the set that'll be checked for collision
// Return Value			:	Handle to the geometry if successful, NULL othervise
// Comments				:
// Date last edited		:	4/15/2001
void	*CCollisionDetector::registerGeometry(CRigidBody *,CArray<CGeometry *> *,int) {
	if (numBodies == maxBodies) {
		if (maxBodies == 0) {
			int	i;
			maxBodies	=	20;

			coefficientsOfFriction				=	new float*[maxBodies];
			coefficientsOfFriction[0]			=	new float[maxBodies*maxBodies];

			coefficientsOfRestitution			=	new float*[maxBodies];
			coefficientsOfRestitution[0]		=	new float[maxBodies*maxBodies];

			for (i=1;i<maxBodies;i++) {
				coefficientsOfFriction[i]		=	&coefficientsOfFriction[0][i*maxBodies];
				coefficientsOfRestitution[i]	=	&coefficientsOfRestitution[0][i*maxBodies];
			}
		} else {
			float	**nf,**nr;
			int		i;

			maxBodies	+=	20;

			nf			=	new float*[maxBodies];
			nf[0]		=	new float[maxBodies*maxBodies];

			nr			=	new float*[maxBodies];
			nr[0]		=	new float[maxBodies*maxBodies];

			for (i=1;i<maxBodies;i++) {
				int	j;

				nf[i]		=	&nf[0][i*maxBodies];
				nr[i]		=	&nr[0][i*maxBodies];

				if (i < numBodies)
					for (j=0;j<numBodies;j++) {
						nf[i][j]	=	coefficientsOfFriction[i][j];
						nr[i][j]	=	coefficientsOfRestitution[i][j];
					}
			}

			delete [] coefficientsOfFriction[0];
			delete [] coefficientsOfFriction;

			delete [] coefficientsOfRestitution[0];
			delete [] coefficientsOfRestitution;

			coefficientsOfFriction		=	nf;
			coefficientsOfRestitution	=	nr;

		}
	}

	{
		int	i;

		for (i=0;i<numBodies;i++) {
			coefficientsOfFriction[numBodies][i]	=	defaultFriction;
			coefficientsOfFriction[i][numBodies]	=	defaultFriction;
			coefficientsOfRestitution[numBodies][i]	=	defaultRestitution;
			coefficientsOfRestitution[i][numBodies]	=	defaultRestitution;
		}
	}

	numBodies++;

	return NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CCollisionDetector
// Method				:	setState
// Description			:	Set the position/orientation of the geometry
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CCollisionDetector::setState(void *handle,float *R,float *T) {
}

///////////////////////////////////////////////////////////////////////
// Class				:	CCollisionDetector
// Method				:	check
// Description			:	Check for a collision
// Return Value			:	TRUE if there is a collision, FALSE othervise
// Comments				:
// Date last edited		:	4/15/2001
int		CCollisionDetector::check() {

	return FALSE;
}

void	CCollisionDetector::computeVelocities(CContact *contact) {
	vector				tmp,tmp2;
	CRigidBody::CState	*state1	=	contact->body1->lastStableState;
	CRigidBody::CState	*state2	=	contact->body2->lastStableState;

	mulmv(tmp,state1->R,contact->P1);
	crossvv(tmp2,state1->avelocity,tmp);
	addvv(contact->vel1,tmp2,state1->velocity);

	mulmv(tmp,state2->R,contact->P2);
	crossvv(tmp2,state2->avelocity,tmp);
	addvv(contact->vel2,tmp2,state2->velocity);

	subvv(contact->vrel,contact->vel1,contact->vel2);

	contact->ndotv	=	dotvv(contact->vrel,contact->normal);
}


// This is used to handle frictionless contact (pretty safe)
void	CCollisionDetector::frictionlessHandler(CContact *contact,float coefficientOfRestitution) {
	float	numerator	=	(float) (-(1 + coefficientOfRestitution)*contact->ndotv);
	float	term1		=	contact->body1->invMass;
	float	term2		=	contact->body2->invMass;
	float	term3,term4;
	float	J;
	vector	r1,r2;
	vector	force;
	vector	tmp,tmp2;
	CRigidBody::CState	*state1	=	contact->body1->lastStableState;
	CRigidBody::CState	*state2	=	contact->body2->lastStableState;

	// Compute the force arms
	mulmv(r1,state1->R,contact->P1);
	mulmv(r2,state2->R,contact->P2);

	// Classical impulse terms
	crossvv(tmp,r1,contact->normal);
	mulmv(tmp2,state1->Iinv,tmp);
	crossvv(tmp,tmp2,r1);
	term3	=	dotvv(contact->normal,tmp);

	crossvv(tmp,r2,contact->normal);
	mulmv(tmp2,state2->Iinv,tmp);
	crossvv(tmp,tmp2,r2);
	term4	=	dotvv(contact->normal,tmp);

	// The normal impulse amount
	J	=	numerator / (term1 + term2 + term3 + term4);

	mulvf(force,contact->normal,J);

	addvv(state1->P,force);
	subvv(state2->P,force);

	crossvv(tmp,r1,force);
	addvv(state1->L,tmp);

	crossvv(tmp,r2,force);
	subvv(state2->L,tmp);

	mulvf(state1->velocity,state1->P,contact->body1->invMass);
	mulvf(state2->velocity,state2->P,contact->body2->invMass);

	mulmv(state1->avelocity,state1->Iinv,state1->L);
	mulmv(state2->avelocity,state2->Iinv,state2->L);
}

// Darn frictionly contact handler
void	CCollisionDetector::handleContact(CContact *contact) {
	vector				tmp,tmp2;
	CRigidBody::CState	*state1	=	contact->body1->lastStableState;
	CRigidBody::CState	*state2	=	contact->body2->lastStableState;
	vector				r1,r2;
	float				J;
	float				coefficientOfFriction		=	coefficientsOfFriction[contact->body1->bodyIndex][contact->body2->bodyIndex];		// Collision constants
	float				coefficientOfRestitution	=	coefficientsOfRestitution[contact->body1->bodyIndex][contact->body2->bodyIndex];	// Collision constants

	if (coefficientOfFriction == 0) {
		// Faster
		frictionlessHandler(contact,coefficientOfRestitution);
		computeVelocities(contact);
		return;
	}
	// Compute the force arms
	mulmv(r1,state1->R,contact->P1);
	mulmv(r2,state2->R,contact->P2);


	{
		vector	u,v;
		float	*n	=	contact->normal;
		matrix	K,KInv,mtmp,mtmp2,mtmp3;				// Compute the collision matrix
		
		vector	impulse;
		float	vdotu,vdotv;

		// Establish a tangent frame to apply the friction force
		crossvv(u,contact->normal,r1);
		normalizev(u);
		crossvv(v,contact->normal,u);

		vdotu	=	dotvv(contact->vrel,u);				// The components of the relative 
		vdotv	=	dotvv(contact->vrel,v);				// velocity on the tangent plane

		// Compute the collision matrix
		identitym(K);
		K[element(0,0)]	*=	(contact->body1->invMass + contact->body2->invMass);
		K[element(1,1)]	*=	(contact->body1->invMass + contact->body2->invMass);
		K[element(2,2)]	*=	(contact->body1->invMass + contact->body2->invMass);

		skewsymm(mtmp2,r1);
		mulmm(mtmp3,state1->Iinv,mtmp2);
		mulmm(mtmp,mtmp2,mtmp3);
		submm(K,mtmp);

		skewsymm(mtmp2,r2);
		mulmm(mtmp3,state2->Iinv,mtmp2);
		mulmm(mtmp,mtmp2,mtmp3);
		submm(K,mtmp);

		K[element(3,3)]	=	1;									// Make sure the matrix is correct

		float	l	=	(float) sqrt(vdotu*vdotu + vdotv*vdotv);
		if (l < 0.000001) {
			movvv(tmp,n);
		} else {
			mulvf(tmp,u,-vdotu*coefficientOfFriction/l);		// Assume Impulse = J*[n + t*u + t*v]
			mulvf(tmp2,v,-vdotv*coefficientOfFriction/l);
			addvv(tmp,tmp2);
			addvv(tmp,n);
		}

		mulmv(tmp2,K,tmp);
		J	=	-(1+coefficientOfRestitution)*dotvv(contact->vrel,n) / dotvv(n,tmp2);

		mulvf(impulse,tmp,J);				// Compute the impulse

											// Apply the impulse
		{
			vector	newP1,newP2,newL1,newL2,newVel1,newVel2,newAVel1,newAVel2;
			vector	vel1,vel2,vrel;

			addvv(newP1,state1->P,impulse);
			subvv(newP2,state2->P,impulse);

			crossvv(tmp,r1,impulse);
			addvv(newL1,state1->L,tmp);

			crossvv(tmp,r2,impulse);
			subvv(newL2,state2->L,tmp);

			mulvf(newVel1,newP1,contact->body1->invMass);
			mulvf(newVel2,newP2,contact->body2->invMass);

			mulmv(newAVel1,state1->Iinv,newL1);
			mulmv(newAVel2,state2->Iinv,newL2);

			crossvv(tmp2,newAVel1,r1);
			addvv(vel1,tmp2,newVel1);

			crossvv(tmp2,newAVel2,r2);
			addvv(vel2,tmp2,newVel2);

			subvv(vrel,vel1,vel2);
			
			{
				float	x,y;

				x	=	dotvv(vrel,u);
				y	=	dotvv(vrel,v);

				if ((x*vdotu + y*vdotv) < 0) {	// The friction force changed the direction of relative motion
												// on the tangent plane, so generate a new impulse
					invertm(KInv,K);

					mulvf(tmp2,n,-(1+coefficientOfRestitution)*dotvv(contact->vrel,n));	// No tangential motion
					mulvf(tmp,u,-vdotu);
					addvv(tmp2,tmp);
					mulvf(tmp,v,-vdotv);
					addvv(tmp2,tmp);

					mulmv(impulse,KInv,tmp2);

					addvv(newP1,state1->P,impulse);
					subvv(newP2,state2->P,impulse);

					crossvv(tmp,r1,impulse);
					addvv(newL1,state1->L,tmp);

					crossvv(tmp,r2,impulse);
					subvv(newL2,state2->L,tmp);

					mulvf(newVel1,newP1,contact->body1->invMass);
					mulvf(newVel2,newP2,contact->body2->invMass);

					mulmv(newAVel1,state1->Iinv,newL1);
					mulmv(newAVel2,state2->Iinv,newL2);

					crossvv(tmp2,newAVel1,r1);
					addvv(vel1,tmp2,newVel1);

					crossvv(tmp2,newAVel2,r2);
					addvv(vel2,tmp2,newVel2);

					subvv(vrel,vel1,vel2);
				}
			}
			if (dotvv(vrel,n) < 0) {
				int	f	=	1;
			}

			// Copy the computed new stuff back
			movvv(state1->P,newP1);
			movvv(state2->P,newP2);

			movvv(state1->L,newL1);
			movvv(state2->L,newL2);

			movvv(state1->velocity,newVel1);
			movvv(state2->velocity,newVel2);

			movvv(state1->avelocity,newAVel1);
			movvv(state2->avelocity,newAVel2);

			contact->ndotv	=	dotvv(vrel,n);

			return;
		}
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSWIFT
// Method				:	CSWIFT
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CSWIFT::CSWIFT() {
	scene			=	new SWIFT_Scene(true);
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSWIFT
// Method				:	~CSWIFT
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CSWIFT::~CSWIFT() {
	delete scene;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSWIFT
// Method				:	registerGeometry
// Description			:	Add a geometry into the set that'll be checked for collision
// Return Value			:	Handle to the geometry if successful, NULL othervise
// Comments				:
// Date last edited		:	4/15/2001
void	*CSWIFT::registerGeometry(CRigidBody *body,CArray<CGeometry *> *g,int fixed) {
	if (g->getSize() == 1) {
		CGeometry	*geo		=	(*g)[0];
		int			id;
		int			i,j;
		int			*faces		=	new int[geo->numTriangles*3];
		SWIFT_Real	*vertices	=	new SWIFT_Real[geo->numVertices*3];

		for (i=0,j=0;i<geo->numTriangles;i++,j+=3) {
			faces[j]	=	geo->triangles[i].v[0];
			faces[j+1]	=	geo->triangles[i].v[1];
			faces[j+2]	=	geo->triangles[i].v[2];
		}

		for (i=0,j=0;i<geo->numVertices;i++,j+=3) {
			vertices[j]		=	geo->vertices[i][COMP_X];
			vertices[j+1]	=	geo->vertices[i][COMP_Y];
			vertices[j+2]	=	geo->vertices[i][COMP_Z];
		}

		scene->Add_Object(vertices,faces,geo->numVertices,geo->numTriangles,id,(fixed ? true : false),DEFAULT_ORIENTATION,DEFAULT_TRANSLATION,DEFAULT_SCALE,DEFAULT_BOX_SETTING,DEFAULT_BOX_ENLARGE_REL,tolerance);

		bodies[id]		=	body;

		CCollisionDetector::registerGeometry(body,g,fixed);

		return (void *) id;
	} else {
		int			numPieces		=	g->getSize();
		SWIFT_Real	**vertices		=	new SWIFT_Real*[numPieces];
		int			**faces			=	new int*[numPieces];
		int			*numVertices	=	new int[numPieces];
		int			*numFaces		=	new int[numPieces];
		int			i;
		SWIFT_Real	*tolerances		=	new SWIFT_Real[numPieces];
		int			id;

		for (i=0;i<numPieces;i++) {
			CGeometry	*geo	=	(*g)[i];
			int			j,k;

			numVertices[i]		=	geo->numVertices;
			numFaces[i]			=	geo->numTriangles;

			vertices[i]			=	new SWIFT_Real[geo->numVertices*3];

			for (k=0,j=0;k<geo->numVertices;k++,j+=3) {
				vertices[i][j]		=	geo->vertices[k][COMP_X];
				vertices[i][j+1]	=	geo->vertices[k][COMP_Y];
				vertices[i][j+2]	=	geo->vertices[k][COMP_Z];
			}

			faces[i]			=	new int[geo->numTriangles*3];

			for (k=0,j=0;k<geo->numTriangles;k++,j+=3) {
				faces[i][j]		=	geo->triangles[k].v[0];
				faces[i][j+1]	=	geo->triangles[k].v[1];
				faces[i][j+2]	=	geo->triangles[k].v[2];
			}

			tolerances[i]	=	tolerance;
		}

		scene->Add_Pieced_Object(vertices,faces,numVertices,numFaces,numPieces,id,(fixed ? true : false),DEFAULT_ORIENTATIONS,DEFAULT_TRANSLATIONS,DEFAULT_SCALES,DEFAULT_BOX_SETTINGS,DEFAULT_BOX_ENLARGE_RELS,tolerances,DEFAULT_FACE_VALENCESS,DEFAULT_COPIES,DEFAULT_COPIES,DEFAULT_MIN_TCOUNTS,DEFAULT_QSLIM_TRATIOS,DEFAULT_CUBE_ASPECT_RATIOS);

		bodies[id]		=	body;

		CCollisionDetector::registerGeometry(body,g,fixed);

		return (void *) id;
	}

	return NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSWIFT
// Method				:	setState
// Description			:	Set the position/orientation of the geometry
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
void	CSWIFT::setState(void *handle,float *R,float *T) {
	SWIFT_Real	Rs[9];
	SWIFT_Real	Ts[3];

	Rs[0]	=	R[element(0,0)];
	Rs[1]	=	R[element(0,1)];
	Rs[2]	=	R[element(0,2)];
	Rs[3]	=	R[element(1,0)];
	Rs[4]	=	R[element(1,1)];
	Rs[5]	=	R[element(1,2)];
	Rs[6]	=	R[element(2,0)];
	Rs[7]	=	R[element(2,1)];
	Rs[8]	=	R[element(2,2)];

	/*
	Rs[0]	=	R[element(0,0)];
	Rs[1]	=	R[element(1,0)];
	Rs[2]	=	R[element(2,0)];
	Rs[3]	=	R[element(0,1)];
	Rs[4]	=	R[element(1,1)];
	Rs[5]	=	R[element(2,1)];
	Rs[6]	=	R[element(0,2)];
	Rs[7]	=	R[element(1,2)];
	Rs[8]	=	R[element(2,2)];
	*/


	Ts[0]	=	T[COMP_X];
	Ts[1]	=	T[COMP_Y];
	Ts[2]	=	T[COMP_Z];

	scene->Set_Object_Transformation((int) handle,Rs,Ts);
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSWIFT
// Method				:	check
// Description			:	Check for a collision
// Return Value			:	TRUE if there is a collision, FALSE othervise
// Comments				:
// Date last edited		:	4/15/2001

int	cheat	=	FALSE;
int		CSWIFT::check() {
	int			numPairs;
	int			i;
	SWIFT_Real	*currentNormal;
	SWIFT_Real	*currentContact;
	int			*currentPair;
	int			*currentFeatures;
	int			*currentFeatureIds;
	SWIFT_Real	*currentDistance;
	
	// If we have interpenetration, we must subdivide
	if (scene->Query_Contact_Determination(false,tolerance,numPairs,&objectPairs,&distances,&contactPoints,&normals,&pointIds,&featureTypes,&featureIds))
		return COLLISION_SUBDIVIDE;

	if (numPairs != 0) {
		currentPair			=	objectPairs;
		currentNormal		=	normals;
		currentFeatures		=	featureTypes;
		currentContact		=	contactPoints;
		currentFeatureIds	=	featureIds;
		currentDistance		=	distances;

		nContacts			=	0;

		for (i=0;i<numPairs;i++) {
			CContact				*contact	=	&contacts[nContacts];

			if (bodies[currentPair[0]] == bodies[currentPair[1]]) {
				currentPair		+=	2;
				currentNormal	+=	3;
				currentFeatures	+=	2;
				currentContact	+=	6;
				currentDistance++;
				continue;
			}

			contact->P1[COMP_X]		=	(float) currentContact[0];
			contact->P1[COMP_Y]		=	(float) currentContact[1];
			contact->P1[COMP_Z]		=	(float) currentContact[2];
			contact->P2[COMP_X]		=	(float) currentContact[3];
			contact->P2[COMP_Y]		=	(float) currentContact[4];
			contact->P2[COMP_Z]		=	(float) currentContact[5];
			contact->normal[COMP_X]	=	(float) currentNormal[0];
			contact->normal[COMP_Y]	=	(float) currentNormal[1];
			contact->normal[COMP_Z]	=	(float) currentNormal[2];

			contact->body1			=	bodies[currentPair[0]];
			contact->body2			=	bodies[currentPair[1]];

			computeVelocities(contact);

			if (contact->ndotv >= 0) {
				currentPair		+=	2;
				currentNormal	+=	3;
				currentFeatures	+=	2;
				currentContact	+=	6;
				currentDistance++;
				continue;
			} else {
				nContacts++;
			}

			currentPair		+=	2;
			currentNormal	+=	3;
			currentFeatures	+=	2;
			currentContact	+=	6;
			currentDistance++;
		} 

		if (nContacts == 0)
			return COLLISION_CLEAR;

		{
			int hadCollision;
			int	count	=	0;

			do {

				hadCollision	=	FALSE;

				for (i=0;i<nContacts;i++) {
					//computeVelocities(&contacts[i]);

					if (contacts[i].ndotv < 0) {
						hadCollision	=	TRUE;
						handleContact(&contacts[i]);
					}
				}

				count ++;

				if (count == 100) {
					int fuck	=	1;
				}

			} while(hadCollision == TRUE);
		}

		return COLLISION_HANDLED;

	}

	return COLLISION_CLEAR;
}


///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	CSimulation
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CSimulation::CSimulation(CCollisionDetector *customDetector) {
	clock				=	0;

	numBodies			=	0;
	maxBodies			=	100;

	bodies				=	new CBodyEntry[maxBodies];

	initv(gravity,0,0,-9.8f);

	maxTimeStep			=	0.01f;
	velocityDamping		=	0.04f;
	avelocityDamping	=	0.01f;

	if (customDetector == NULL)
		collisionDetector	=	new CCollisionDetector();
	else
		collisionDetector	=	customDetector;

	psystems				=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	~CSimulation
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	4/15/2001
CSimulation::~CSimulation() {
	delete [] bodies;
	delete collisionDetector;

	if (psystems != NULL)	delete psystems;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	addObject
// Description			:	Add an object into the simulation
// Return Value			:
// Comments				:
// Date last edited		:	4/15/2001
void	CSimulation::addObject(CRigidBody *object) {

	if (numBodies == maxBodies) {
		CBodyEntry	*newBodies;
		int			i;

		maxBodies	+=	100;

		newBodies	=	new CBodyEntry[maxBodies];

		for (i=0;i<numBodies;i++)
			newBodies[i]	=	bodies[i];

		delete [] bodies;
		bodies	=	newBodies;
	}

	bodies[numBodies].body		=	object;
	mcpy(&bodies[numBodies].initialState,object->lastStableState,sizeof(CRigidBody::CState));

	numBodies++;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	addPsystem
// Description			:	Add a particle system into the simulation
// Return Value			:
// Comments				:
// Date last edited		:	4/15/2001
void	CSimulation::addPsystem(CParticleSystem *system) {
	system->next	=	psystems;
	psystems		=	system;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	simulate
// Description			:	Integrate the equations of motion for 'dtime'
// Return Value			:
// Comments				:
// Date last edited		:	4/15/2001
int	c	=	1;
void	CSimulation::simulate(float dtime) {
	int			i;

	while(dtime > maxTimeStep) {
		step(maxTimeStep);

		if (psystems != NULL) {
			psystems->step(maxTimeStep);
		}

		advance(maxTimeStep);
		dtime	-=	maxTimeStep;
	}

	if (dtime > 0)	{
		step(dtime);

		if (psystems != NULL) {
			psystems->step(dtime);
		}

		advance(dtime);
	}

	for (i=0;i<numBodies;i++) {
		int	j;

		for (j=0;j<bodies[i].body->geometry->getSize();j++) {
			geometry((*bodies[i].body->geometry)[j],bodies[i].body->lastStableState->R,bodies[i].body->lastStableState->position);
			//geometry(bodies[i].body->geometry,bodies[i].body->currentState->R,bodies[i].body->currentState->position);
		}
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	reset
// Description			:	Reset the simulation to the initial state
// Return Value			:
// Comments				:
// Date last edited		:	4/17/2001
void	CSimulation::reset() {
	int	i;

	for (i=0;i<numBodies;i++) {
		mcpy(bodies[i].body->lastStableState,&bodies[i].initialState,sizeof(CRigidBody::CState));
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSimulation
// Method				:	addForces
// Description			:	This function is called for each body for
//							each time step for accumulating the forces
//							on the body. By default, it just adds the gravity
// Return Value			:	The list of forces on the body, NULL if none
// Comments				:
// Date last edited		:	4/15/2001
void	CSimulation::addForces(CRigidBody *body) {
	if (!(body->flags & RBODY_FIXED)) {
		vector	tmp;

		mulvf(body->netForce,gravity,body->mass);
		initv(body->netTorque,0,0,0);

		if (psystems != NULL) {
			psystems->collide(body);
		}

		mulvf(tmp,body->lastStableState->velocity,-velocityDamping);
		addvv(body->netForce,tmp);

		mulvf(tmp,body->lastStableState->avelocity,-avelocityDamping);
		addvv(body->netTorque,tmp);
	} else {
		initv(body->netForce,0,0,0);
		initv(body->netTorque,0,0,0);
	}
}

void	CSimulation::advance(float dtime) {
	int			i;
	int			collision		=	0;
	float		timeStart		=	0;
	float		timeEnd			=	dtime;

	int			count			=	2000;

	while (timeEnd > timeStart) {	

		count--;
		if (count == 0) {
			count--;
		}

		// Accumulate forces
		for (i=0;i<numBodies;i++) {
			addForces(bodies[i].body);
		}

		// Integrate the simulation
		for (i=0;i<numBodies;i++) {
			bodies[i].body->advance(timeEnd - timeStart);
			collisionDetector->setState(bodies[i].body->collisionHandle,bodies[i].body->currentState->R,bodies[i].body->currentState->position);
		}

		// Collision detection
		switch(collisionDetector->check()) {
		case COLLISION_CLEAR:
			// At this point, the current state must be free of any collision
			for (i=0;i<numBodies;i++) {
				bodies[i].body->commit();
			}

			timeStart	=	timeEnd;
			timeEnd		=	dtime;
			break;
		case COLLISION_HANDLED:
			// At this point, we just had a collision and the collision detector
			// modified the last stable state to avoid the collision. So I should
			// fall back and re-integrate from the last stable state again until
			// I'm free of any collisions. Note that this step may create very long
			// recursions

			break;
		case COLLISION_SUBDIVIDE:
			// I'm having a collision and the penetration is too much. Subdivide the time
			// interval to integrate
			timeEnd		=	(timeStart + timeEnd) / 2;
			break;
		}
	}


	clock	+=	dtime;
}