%{
#undef		alloca
#include	"mySimulation.h"

void	mesherror(char *);
int		meshlex(void );					// Forward definition for stupid yacc

static	int	numMaterials	=	0;
static	int	numVertices		=	0;
static	int	numTriangles	=	0;
static	int	currentVertex	=	0;
static	int	currentTriangle	=	0;

static	CMyGeometry::CMaterial			*materials		=	NULL;
static	CGeometry::CTriangle			*triangles		=	NULL;
static	CMyGeometry::CMyTriangle		*triangleDatas	=	NULL;
static	vector							*vertices		=	NULL;

%}
%union {
  char			string[64];
  float			real;
}
%token			MESH_DIFFUSE
%token			MESH_SPECULAR
%token			MESH_TEXTURE
%token			MESH_COLOR
%token			MESH_MATERIAL
%token			MESH_VERTICES
%token			MESH_TRIANGLES
%token			MESH_MATERIALS
%token			MESH_SCALE
%token			MESH_ROTATE
%token			MESH_TRANSLATE
%token			MESH_OPEN
%token			MESH_CLOSE
%token<string>	MESH_TEXT
%token<real>	MESH_FLOAT
%%

start:			meshFile;

meshFile:		meshBlock
				meshFile
				{
				}
				|
				{
				}
				;

meshBlock:		meshVerticesBlock
				|
				meshTrianglesBlock
				|
				meshMaterialsBlock
				;

meshVerticesBlock:	MESH_FLOAT
					MESH_VERTICES
					{
						numVertices		=	(int) $1;
						vertices		=	new vector[numVertices];
						currentVertex	=	0;
					}
					meshVertices
					{
					}
					;

meshVertices:		meshVertex
					meshVertices
					{
					}
					|
					{
					}
					;

meshVertex:			MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						vertices[currentVertex][COMP_X]	=	$2;
						vertices[currentVertex][COMP_Y]	=	$3;
						vertices[currentVertex][COMP_Z]	=	$4;
						currentVertex++;
					}

meshTrianglesBlock:	MESH_FLOAT
					MESH_TRIANGLES
					{
						numTriangles	=	(int) $1;
						triangles		=	new CGeometry::CTriangle[numTriangles];
						triangleDatas	=	new CMyGeometry::CMyTriangle[numTriangles];
						currentTriangle	=	0;
					}
					meshTriangles
					{
					}
					;

meshTriangles:		meshTriangle
					meshTriangles
					{
					}
					|
					{
					}
					;
meshTriangle:		MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						triangles[currentTriangle].v[0]				=	(int) $2;
						triangles[currentTriangle].v[1]				=	(int) $3;
						triangles[currentTriangle].v[2]				=	(int) $4;
						triangleDatas[currentTriangle].materialID	=	(int) $5;
						currentTriangle++;
					}
					|
					MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						triangles[currentTriangle].v[0]				=	(int) $2;
						triangles[currentTriangle].v[1]				=	(int) $3;
						triangles[currentTriangle].v[2]				=	(int) $4;
						triangleDatas[currentTriangle].materialID	=	(int) $5;
						triangleDatas[currentTriangle].uCoord[0]	=	$6;
						triangleDatas[currentTriangle].vCoord[0]	=	$7;
						triangleDatas[currentTriangle].uCoord[1]	=	$8;
						triangleDatas[currentTriangle].vCoord[1]	=	$9;
						triangleDatas[currentTriangle].uCoord[2]	=	$10;
						triangleDatas[currentTriangle].vCoord[2]	=	$11;
						currentTriangle++;
					}
					;

meshMaterialsBlock:	MESH_FLOAT
					MESH_MATERIALS
					{
						numMaterials	=	(int) $1;
						materials		=	new CMyGeometry::CMaterial[numMaterials];
					}
					meshMaterialSpecs
					;

meshMaterialSpecs:	
					meshMaterialSpecs
					meshMaterialSpec
					|
					;

meshMaterialSpec:	
					MESH_TEXTURE
					MESH_FLOAT
					MESH_OPEN
					MESH_TEXT
					MESH_CLOSE
					{
						int	i	=	(int) $2;

						materials[i].textureID	=	CMyGeometry::CMaterial::loadTexture($4);

						if (materials[i].textureID >= 0)
							materials[i].materialFlags	|=	MATERIAL_TEXTUREMAPPED;

					}
					|
					MESH_COLOR
					MESH_FLOAT
					MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						int	i						=	(int) $2;
						materials[i].color[COMP_R]	=	$4;
						materials[i].color[COMP_G]	=	$5;
						materials[i].color[COMP_B]	=	$6;
					}
					|
					MESH_DIFFUSE
					MESH_FLOAT
					MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						int	i							=	(int) $2;
						materials[i].diffuse[COMP_R]	=	$4;
						materials[i].diffuse[COMP_G]	=	$5;
						materials[i].diffuse[COMP_B]	=	$6;
					}
					|
					MESH_SPECULAR
					MESH_FLOAT
					MESH_OPEN
					MESH_FLOAT
					MESH_FLOAT
					MESH_FLOAT
					MESH_CLOSE
					{
						int	i							=	(int) $2;
						materials[i].specular[COMP_R]	=	$4;
						materials[i].specular[COMP_G]	=	$5;
						materials[i].specular[COMP_B]	=	$6;
					}
					;
%%



#include "lex.mesh.cpp"

int		lineNo		=	0;
int		numErrors	=	0;
CMyGeometry	*parseGeometry(const char *fileName) {
	char	location[OS_MAX_PATH_LENGTH];
	
	if (COs::locateFile(location,fileName) == FALSE)	return NULL;

	vertices		=	NULL;
	triangles		=	NULL;
	triangleDatas	=	NULL;
	materials		=	NULL;

	if (location != NULL) {
		yyin	=	fopen(location,"r");
		if (yyin != NULL) {
			lineNo		=	1;
			numErrors	=	0;

			meshparse();

			if (numErrors == 0) {
				CMyGeometry	*geometry	=	new CMyGeometry;
				geometry->vertices		=	vertices;
				geometry->triangles		=	triangles;
				geometry->triangleDatas	=	triangleDatas;
				geometry->materials		=	materials;

				geometry->numVertices	=	numVertices;
				geometry->numTriangles	=	numTriangles;
				geometry->numMaterials	=	numMaterials;

				geometry->prepare();
				return geometry;
			}
		}
	}

	if (vertices != NULL)		delete [] vertices;
	if (materials != NULL)		delete [] materials;
	if (triangles != NULL)		delete [] triangles;
	if (triangleDatas != NULL)	delete [] triangleDatas;
	return NULL;
}

void	mesherror(char *str) {
	numErrors++;
}