// mySimulation.cpp: implementation of the mySimulation class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "mySimulation.h"
#include "gl\glu.h"
#include <tiff.h>
#include <tiffio.h>

#include "common/opengl.h"

typedef struct {
	unsigned int		id;
	char				name[64];
	int					width,height,numSamples;
} CTexture;

static	CTexture	*textures	=	NULL;
static	int			nTextures	=	0;
static	int			mTextures	=	0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
extern char inputPath[1024];
int	CMyGeometry::CMaterial::loadTexture(char *name) {
	int		i;
	char	*type	=	"trilinear";
	char	tmp[256];

	for (i=0;i<nTextures;i++) {
		if (scmp(textures[i].name,name) == 0)	return textures[i].id;
	}

	if (schr(name,'.') == NULL) {
		sprintf(tmp,"%s.tif",name);
	} else {
		sprintf(tmp,"%s",name);
	}

	if (nTextures == mTextures) {
		if (mTextures == 0) {
			mTextures	=	10;
			textures	=	new CTexture[mTextures];
		} else {
			CTexture	*newTextures	=	new CTexture[mTextures+10];
			mcpy(newTextures,textures,nTextures*sizeof(CTexture));
			delete [] textures;
			textures	=	newTextures;

			mTextures+=10;
		}
	}

	CTexture	*cTexture	=	&textures[nTextures++];
	char		fn[OS_MAX_PATH_LENGTH];

	if (COs::locateFile(fn,tmp,inputPath) == FALSE) {
	} else {
		// Set the error handler so we don't crash
		//TIFFSetErrorHandler(errorHandler);
		//TIFFSetWarningHandler(errorHandler);

		TIFF			*in		=	TIFFOpen(fn,"r");
		unsigned char	*udata;
		int				i;
		uint32			w,h;
		uint16			ns,bp,organization;

		if (in == NULL)	return NULL;

		// Set the necessary stuff
		TIFFGetFieldDefaulted(in, TIFFTAG_IMAGEWIDTH,			&w);
		TIFFGetFieldDefaulted(in, TIFFTAG_IMAGELENGTH,			&h);
		TIFFGetFieldDefaulted(in, TIFFTAG_SAMPLESPERPIXEL,		&ns);
		TIFFGetFieldDefaulted(in, TIFFTAG_BITSPERSAMPLE,		&bp);
		TIFFGetFieldDefaulted(in, TIFFTAG_SAMPLEFORMAT,			&organization);

		udata	=	new unsigned char[w*h*ns];

		for (i=0;i<(int) h;i++) {
			TIFFReadScanline(in,&udata[i*w*ns],i,0);
		}

		TIFFClose(in);


		cTexture->width			=	w;
		cTexture->height		=	h;
		cTexture->numSamples	=	ns;
		scpy(cTexture->name,name);
		glGenTextures(1,&cTexture->id);

		glBindTexture(GL_TEXTURE_2D,cTexture->id);

		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);  
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		GLenum	channelTypeGL;
		GLenum	sampleTypeGL;

		switch(ns) {
		case 1:
			channelTypeGL	=	GL_LUMINANCE;
			break;
		case 2:
			channelTypeGL	=	GL_LUMINANCE_ALPHA;
			break;
		case 3:
			channelTypeGL	=	GL_RGB;
			break;
		case 4:
			channelTypeGL	=	GL_RGBA;
			break;
		}

		sampleTypeGL	=	GL_UNSIGNED_BYTE;

		if (scmp(type,"bilinear") == 0) {
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D,0,ns,w,h,0,channelTypeGL,sampleTypeGL,udata);	
		} else if (scmp(type,"trilinear") == 0) {
			gluBuild2DMipmaps(GL_TEXTURE_2D,3, w,h,channelTypeGL,sampleTypeGL,udata);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
       		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		} else {
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexImage2D(GL_TEXTURE_2D,0,ns,w,h,0,channelTypeGL,sampleTypeGL,udata);	
		}

		delete [] udata;
	}
	
	
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 

	return cTexture->id;
}


void	CMyGeometry::draw() {
	glCallList(listItem);
}

void	CMyGeometry::prepare()  {
	int			i;
	CMaterial	*material;
	CTriangle	*triangle;
	CMyTriangle	*triangleData;

	listItem	=	glGenLists(1);

	glNewList(listItem,GL_COMPILE);

	
	for (i=0;i<numTriangles;i++) {
		material	=	&materials[triangleDatas[i].materialID];

		triangle		=	&triangles[i];
		triangleData	=	&triangleDatas[i];

		glColor3fv(material->color);

		glNormal3fv(triangle->norm);
		if (material->materialFlags & MATERIAL_TEXTUREMAPPED) {
			glBindTexture(GL_TEXTURE_2D,material->textureID);

			glBegin(GL_TRIANGLES);

			glTexCoord2f(triangleData->uCoord[0],triangleData->vCoord[0]);
			glVertex3fv(vertices[triangle->v[0]]);

			glTexCoord2f(triangleData->uCoord[1],triangleData->vCoord[1]);
			glVertex3fv(vertices[triangle->v[1]]);
			
			glTexCoord2f(triangleData->uCoord[2],triangleData->vCoord[2]);
			glVertex3fv(vertices[triangle->v[2]]);

			glEnd();
		} else {
			glDisable(GL_TEXTURE_2D);
			glBegin(GL_TRIANGLES);
			glVertex3fv(vertices[triangle->v[0]]);
			glVertex3fv(vertices[triangle->v[1]]);
			glVertex3fv(vertices[triangle->v[2]]);
			glEnd();
			glEnable(GL_TEXTURE_2D);
		}
	}

	glEndList();
}

// Use swift for collision detection
CMySimulation::CMySimulation() : CSimulation(new CSWIFT) {
 	mLights		=	10;
	nLights		=	0;
	lights		=	new CLight[mLights];

	refreshRate	=	0.033333333f;
}

CMySimulation::~CMySimulation() {
	delete [] lights;
}

void	CMySimulation::draw(float *toCamera,float *fromCamera,float tx,float ty,float tz,float dtime) {
	const	unsigned char	*c	=	glGetString(GL_VENDOR);
	c	=	glGetString(GL_RENDERER);

	glPushMatrix();

	glCallList(globalList);

	glLoadIdentity();
	glTranslatef(tx,ty,tz);
	glMultMatrixf(toCamera);
	simulate(dtime);
	glPopMatrix();

	if (psystems != NULL) {
		psystems->draw(toCamera,fromCamera,tx,ty,tz,dtime);
	}
}

void	CMySimulation::step(float dtime)  {
}

void	CMySimulation::geometry(CGeometry *g,float *R,float *P) {
	CMyGeometry		*geometry	=	(CMyGeometry *) g;

	glPushMatrix();
	glTranslatef(P[COMP_X],P[COMP_Y],P[COMP_Z]);
	glMultMatrixf(R);
	geometry->draw();
	glPopMatrix();
}

void	CMySimulation::addLight(CLight *light) {
	if (nLights >= mLights) return;

	mcpy(&lights[nLights++],light,sizeof(CLight));
}

void	CMySimulation::prepare() {
	int	i;

	globalList	=	glGenLists(1);

	glNewList(globalList,GL_COMPILE);

	for (i=0;i<nLights;i++) {
		GLenum	lightNumber;
		float	data[4];

		switch(i) {
		case 0:
			lightNumber	=	GL_LIGHT0;
			break;
		case 1:
			lightNumber	=	GL_LIGHT1;
			break;
		case 2:
			lightNumber	=	GL_LIGHT2;
			break;
		case 3:
			lightNumber	=	GL_LIGHT3;
			break;
		case 4:
			lightNumber	=	GL_LIGHT4;
			break;
		case 5:
			lightNumber	=	GL_LIGHT5;
			break;
		case 6:
			lightNumber	=	GL_LIGHT6;
			break;
		case 7:
			lightNumber	=	GL_LIGHT7;
			break;
		default:
			lightNumber	=	GL_LIGHT7;
			break;
		}

		data[0]	=	lights[i].ambient[COMP_R];
		data[1]	=	lights[i].ambient[COMP_G];
		data[2]	=	lights[i].ambient[COMP_B];
		data[3]	=	1;
		glLightfv(lightNumber,GL_AMBIENT,data);

		data[0]	=	lights[i].color[COMP_R];
		data[1]	=	lights[i].color[COMP_G];
		data[2]	=	lights[i].color[COMP_B];
		data[3]	=	1;
		glLightfv(lightNumber,GL_DIFFUSE,data);

		data[0]	=	1;
		data[1]	=	1;
		data[2]	=	1;
		data[3]	=	1;
		glLightfv(lightNumber,GL_SPECULAR,data);

		data[0]	=	lights[i].dir[COMP_X];
		data[1]	=	lights[i].dir[COMP_Y];
		data[2]	=	lights[i].dir[COMP_Z];
		data[3]	=	lights[i].w;
		glLightfv(lightNumber,GL_POSITION,data);

		glEnable(lightNumber);
	}

	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_CULL_FACE);
    glEnable(GL_DEPTH_TEST);
    glShadeModel(GL_FLAT);
	glEnable(GL_TEXTURE_2D);

	glEndList();
}