#include "psystem.h"
#include "simulation.h"
#include "common/os.h"
#include "common/opengl.h"

CParticle	*append(CParticle *p1,CParticle *p2) {
	CParticle	*cParticle;

	if (p1 == NULL)	return	p2;

	for (cParticle=p1;cParticle->next != NULL;cParticle=cParticle->next);
	cParticle->next	=	p2;

	return p1;
}

CParticle	*newParticle(CParticle *&freeParticles) {
	CParticle	*cParticle	=	freeParticles;

	if (cParticle == NULL) {
		cParticle		=	new CParticle;
	} else {
		freeParticles	=	cParticle->next;
	}

	return cParticle;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CEmitter
// Method				:	CEmitter
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CEmitter::CEmitter(float *transform,float particlesPerSecond,float particleLifeSpan,float particleMass,float speed,float angleVariance,float speedVariance) {
	vector	tmp;

	this->particlesPerSecond	=	particlesPerSecond;
	this->particleLifeSpan		=	particleLifeSpan;
	this->particleMass			=	particleMass;
	this->speed					=	speed;
	this->angleVariance			=	angleVariance;
	this->speedVariance			=	speedVariance;

	this->next					=	NULL;

	initv(tmp,0,0,0);
	mulmv(source,transform,tmp);

	initv(tmp,0,0,1);
	mulmv(dir,transform,tmp);
	subvv(dir,source);

	numLeftoverParticles		=	0;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CEmitter
// Method				:	~CEmitter
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CEmitter::~CEmitter() {
	if (next != NULL) {
		delete next;
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CEmitter
// Method				:	emit
// Description			:	Emit particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
CParticle	*CEmitter::emit(float dtime,CParticle *&particles) {
	int			numParticles	=	(int) (dtime*this->particlesPerSecond + numLeftoverParticles);
	int			i;
	CParticle	*nParticles		=	NULL;
	vector		dx,dy;
	vector		t;

	// Create an orthagonal coordinate system
	initv(t,1,3,8);
	crossvv(dx,dir,t);
	crossvv(dy,dx,dir);

	normalizev(dx);
	normalizev(dy);

	numLeftoverParticles		=	(dtime*this->particlesPerSecond + numLeftoverParticles - (float) numParticles);

	for (i=0;i<numParticles;i++) {
		CParticle	*cParticle	=	newParticle(particles);
		float		speed		=	rand() / (float) RAND_MAX;
		float		dr			=	rand() / (float) RAND_MAX;
		float		dtheta		=	rand() / (float) RAND_MAX;
		vector		tdx,tdy;
		vector		d;

		dr			*=	(float) tan(radians(this->angleVariance));
		mulvf(tdx,dx,(float) cos(dtheta*2*PI)*dr);
		mulvf(tdy,dy,(float) sin(dtheta*2*PI)*dr);

		speed		+=	this->speed;

		movvv(cParticle->position,this->source);

		movvv(d,this->dir);
		addvv(d,tdx);
		addvv(d,tdy);

		normalizev(d);
		mulvf(cParticle->speed,d,speed);

		cParticle->life	=	this->particleLifeSpan;
		cParticle->mass	=	this->particleMass;
		cParticle->next	=	nParticles;
		nParticles		=	cParticle;
	}

	if (next != NULL) {
		return append(nParticles,next->emit(dtime,particles));
	}

	return nParticles;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CEmitter
// Method				:	clone
// Description			:	Create an exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CEmitter	*CEmitter::clone() {
	CEmitter	*cEmitter	=	new CEmitter;

	cEmitter[0]				=	this[0];

	if (next != NULL)
		cEmitter->next			=	next->clone();
	else
		cEmitter->next			=	NULL;

	return cEmitter;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSemitter
// Method				:	CSemitter
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSemitter::CSemitter(float *transform,float particleLifeSpan,float particleMass,float speed) : CEmitter(transform,1,particleLifeSpan,particleMass,speed,0,0) {
	emitted	=	FALSE;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSemitter
// Method				:	~CSemitter
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSemitter::~CSemitter() {
	if (next != NULL) {
		delete next;
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CSemitter
// Method				:	emit
// Description			:	Emit particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
CParticle	*CSemitter::emit(float dtime,CParticle *&particles) {
	if (emitted == FALSE) {
		CParticle	*nParticle	=	newParticle(particles);

		emitted	=	TRUE;

		movvv(nParticle->position,source);
		mulvf(nParticle->speed,dir,speed);
		nParticle->life	=	this->particleLifeSpan;
		nParticle->mass	=	this->particleMass;
		nParticle->next	=	NULL;

		if (next != NULL) {
			return append(nParticle,next->emit(dtime,particles));
		}

		return nParticle;
	}

	return NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSemitter
// Method				:	clone
// Description			:	Create an exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CEmitter	*CSemitter::clone() {
	CSemitter	*cEmitter	=	new CSemitter;

	cEmitter[0]				=	this[0];

	if (next != NULL)
		cEmitter->next			=	next->clone();
	else
		cEmitter->next			=	NULL;

	return cEmitter;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	CForceSource
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource::CForceSource(float *transform) {
	this->next	=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	~CForceSource
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource::~CForceSource() {
	if (next != NULL)	delete next;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	force
// Description			:	Add forces to particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
void	CForceSource::force(float dtime,CParticle *particles) {
	if (next != NULL) {
		next->force(dtime,particles);
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	clone
// Description			:	Clone exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource *CForceSource::clone() {
	CForceSource	*cForce	=	new CForceSource;
	
	cForce[0]				=	this[0];

	if (next != NULL)
		cForce->next		=	next->clone();
	else
		cForce->next		=	NULL;

	return cForce;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CGravity
// Method				:	CGravity
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CGravity::CGravity(float *transform,float g) {
	vector	o,tmp;

	initv(tmp,0,0,0);
	mulmv(o,transform,tmp);

	initv(tmp,0,0,-1);
	mulmv(dir,transform,tmp);
	subvv(dir,o);

	this->g		=	g;
	this->next	=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	~CForceSource
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CGravity::~CGravity() {
	if (next != NULL)	delete next;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	force
// Description			:	Add forces to particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
void	CGravity::force(float dtime,CParticle *particles) {
	CParticle	*cParticle;

	for (cParticle=particles;cParticle!=NULL;cParticle=cParticle->next) {
		vector	f;

		mulvf(f,dir,cParticle->mass*g);
		addvv(cParticle->force,f);
	}

	if (next != NULL) {
		next->force(dtime,particles);
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Method				:	clone
// Description			:	Clone exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource *CGravity::clone() {
	CGravity	*cForce	=	new CGravity;
	
	cForce[0]				=	this[0];

	if (next != NULL)
		cForce->next		=	next->clone();
	else
		cForce->next		=	NULL;

	return cForce;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Method				:	CSwirl
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSwirl::CSwirl(float *transform,float m) {
	vector	tmp;

	initv(tmp,0,0,0);
	mulmv(o,transform,tmp);

	initv(tmp,0,0,1);
	mulmv(z,transform,tmp);
	subvv(z,o);

	initv(tmp,1,0,0);
	mulmv(x,transform,tmp);
	subvv(x,o);

	initv(tmp,0,1,0);
	mulmv(y,transform,tmp);
	subvv(y,o);

	this->magnitude	=	m;
	this->next		=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Method				:	~CSwirl
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSwirl::~CSwirl() {
	if (next != NULL)	delete next;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Method				:	force
// Description			:	Add forces to particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
void	CSwirl::force(float dtime,CParticle *particles) {
	CParticle	*cParticle;

	for (cParticle=particles;cParticle!=NULL;cParticle=cParticle->next) {
		vector	f,tmp;
		vector	p;
		float	xp;
		float	yp;
		float	xf,yf,l;

		subvv(p,cParticle->position,o);
		xp		=	dotvv(p,x);
		yp		=	dotvv(p,y);
		l		=	(float) sqrt(xp*xp + yp*yp);
		if (l < EPSILON) continue;
		xp		/=	l;
		yp		/=	l;
		xp		*=	magnitude;
		yp		*=	magnitude;

		subvv(tmp,cParticle->position,o);

		if (dotvv(tmp,cParticle->speed) > 0) {
			xf	=	yp;
			yf	=	-xp;
		} else {
			xf	=	-yp;
			yf	=	xp;
		}

		mulvf(f,x,xf);
		mulvf(tmp,y,yf);
		addvv(f,tmp);
		addvv(cParticle->force,f);
	}

	if (next != NULL) {
		next->force(dtime,particles);
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Method				:	clone
// Description			:	Clone exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource *CSwirl::clone() {
	CSwirl	*cForce	=	new CSwirl;
	
	cForce[0]				=	this[0];

	if (next != NULL)
		cForce->next		=	next->clone();
	else
		cForce->next		=	NULL;

	return cForce;
}


///////////////////////////////////////////////////////////////////////
// Class				:	CSink
// Method				:	CSink
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSink::CSink(float *transform,float m) {
	vector	tmp;

	initv(tmp,0,0,0);
	mulmv(src,transform,tmp);

	this->magnitude	=	m;
	this->next		=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSink
// Method				:	~CSink
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CSink::~CSink() {
	if (next != NULL)	delete next;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSink
// Method				:	computeF
// Description			:	Comnpute the F function
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
void	CSink::computeF(float *F,float *position,float *speed,float mass) {
	vector	f;
	float	l;

	if (FALSE) {
		subvv(f,src,position);
		l	=	lengthv(f);
		if (l > EPSILON) {
			mulvf(F,f,magnitude/(l*mass));
		} else {
			initv(F,0,0,0);
		}
	} else {
		subvv(f,src,position);
		mulvf(F,f,magnitude/(mass));
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CSink
// Method				:	force
// Description			:	Add forces to particles
// Return Value			:
// Comments				:
// Date last edited		:	11/16/2001
void	CSink::force(float dtime,CParticle *particles) {
	CParticle	*cParticle;

	for (cParticle=particles;cParticle!=NULL;cParticle=cParticle->next) {
		vector	f;

		computeF(f,cParticle->position,cParticle->speed,cParticle->mass);
		mulvf(cParticle->force,f,cParticle->mass);
	}

	if (next != NULL) {
		next->force(dtime,particles);
	}
}


///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Method				:	clone
// Description			:	Clone exact replica
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CForceSource *CSink::clone() {
	CSink	*cForce	=	new CSink;
	
	cForce[0]				=	this[0];

	if (next != NULL)
		cForce->next		=	next->clone();
	else
		cForce->next		=	NULL;

	return cForce;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Method				:	CParticleSystem
// Description			:	Ctor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CParticleSystem::CParticleSystem(CEmitter *emitters,CForceSource *forcers,float *color,char *out) {
	unsigned char	*texture	=	new unsigned char[32*32*4];
	int		i,j;

	//integration	=	INTEGRATE_FEULER;
	integration	=	INTEGRATE_RK4;

	if (out != NULL) {
		outFile	=	fopen(out,"w");
	} else {
		outFile	=	NULL;
	}

	for (i=0;i<32;i++) {
		for (j=0;j<32;j++) {
			float	x	=	(float) (i - 16);
			float	y	=	(float) (j - 16);
			float	r	=	x*x + y*y;
			float	c	=	(float) exp(-r*0.05);
			texture[(i*32+j)*4+0]	=	(unsigned char) (color[0]*255);
			texture[(i*32+j)*4+1]	=	(unsigned char) (color[1]*255);
			texture[(i*32+j)*4+2]	=	(unsigned char) (color[2]*255);
			texture[(i*32+j)*4+3]	=	(unsigned char) min((int) (c*256),255);
		}
	}

	glGenTextures(1,&ptexture);

	glBindTexture(GL_TEXTURE_2D,ptexture);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);  
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D,0,4,32,32,0,GL_RGBA,GL_UNSIGNED_BYTE,texture);	
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 

	delete [] texture;

	this->emitters	=	emitters;
	this->forcers	=	forcers;

	particles		=	NULL;
	freeParticles	=	NULL;

	next			=	NULL;
}

///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Method				:	~CParticleSystem
// Description			:	Dtor
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
CParticleSystem::~CParticleSystem() {
	if (outFile != NULL) {
		fclose(outFile);
	}

	if (emitters	!= NULL)	delete emitters;
	if (forcers		!=	NULL)	delete forcers;

	if (particles != NULL) {
		CParticle	*cParticle,*nParticle;

		for (cParticle=particles;cParticle!=NULL;) {
			nParticle	=	cParticle->next;
			delete cParticle;
			cParticle	=	nParticle;
		}
	}

	if (freeParticles != NULL) {
		CParticle	*cParticle,*nParticle;

		for (cParticle=particles;cParticle!=NULL;) {
			nParticle	=	cParticle->next;
			delete cParticle;
			cParticle	=	nParticle;
		}
	}

	if (next != NULL)	delete next;
}


///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Method				:	collide
// Description			:	Resolve collisions with a rigid body
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
void		CParticleSystem::collide(CRigidBody *body) {
	CParticle	*cParticle;
	int			i;

	if (!(body->flags & RBODY_FIXED)) {
		for (cParticle=particles;cParticle!=NULL;cParticle=cParticle->next) {
			// Consider this particle
			vector					pos,speed,tmp;
			CGeometry				*cGeometry;
			float					minPenetration	=	-INFINITY;
			CGeometry::CTriangle	*minTriangle	=	NULL;

			// Convert the particle to the object system
			subvv(tmp,cParticle->position,body->currentState->position);
			mulmv(pos,body->currentState->Rt,tmp);
			mulmv(speed,body->currentState->Rt,cParticle->speed);

			// Find out if we're inside the body
			cGeometry	=	(*body->geometry)[0];
			// FIXME: I need to iterate through all geometries

			for (i=0;i<cGeometry->numTriangles;i++) {
				CGeometry::CTriangle	*cTriangle	=	&cGeometry->triangles[i];
				float					penetration;

				penetration	=	dotvv(pos,cTriangle->norm) + cTriangle->d;

				if (penetration > 0)					break;
				if (dotvv(speed,cTriangle->norm) > 0)	continue;
				if (penetration > minPenetration) {
					minPenetration	=	penetration;
					minTriangle		=	cTriangle;
				}
			}

			if (i == cGeometry->numTriangles) {	// Particle is colliding with the object
				// The colliding triangle is minTriangle , reflect the particle around the triangle
				vector	impulse;

				mulvf(impulse,minTriangle->norm,-2*dotvv(minTriangle->norm,speed));
				addvv(speed,impulse);

				mulmv(cParticle->position,body->currentState->R,pos);
				addvv(cParticle->position,body->currentState->position);
				mulmv(cParticle->speed,body->currentState->R,speed);

				// Apply an opposite impulse to the body
				mulvf(impulse,-cParticle->mass);

				addvv(body->netForce,impulse);
				crossvv(tmp,pos,impulse);
				addvv(body->netTorque,tmp);
			}
		}
	}
	
	if (next != NULL) {
		next->collide(body);
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Method				:	step
// Description			:	Take a step
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
void		CParticleSystem::step(float dtime) {
	CParticle	*cParticle;

	if (outFile != NULL) {	// Compute enery of each particle
		float	Ek;
		float	Ep;
		float	Et;

		for (cParticle=particles;cParticle != NULL;cParticle=cParticle->next) {
			CSink	*cSink	=	(CSink *) forcers;
			vector	tmp;

			subvv(tmp,cParticle->position,cSink->src);

			if (FALSE) {
				Ek				=	dotvv(cParticle->speed,cParticle->speed)*cParticle->mass / (float) 2;
				Ep				=	lengthv(tmp)*cSink->magnitude;
				Et				=	Ek + Ep;
			} else {
				Ek				=	dotvv(cParticle->speed,cParticle->speed)*cParticle->mass / (float) 2;
				Ep				=	dotvv(tmp,tmp)*cSink->magnitude/(float) 2;
				Et				=	Ek + Ep;
			}

			fprintf(outFile,"%f %f %f\n",Ek,Ep,Et);
		}
	}

	if (emitters != NULL)
		particles	=	append(particles,emitters->emit(dtime,freeParticles));


	if (integration == INTEGRATE_FEULER) {
		for (cParticle=particles;cParticle != NULL;cParticle=cParticle->next) {
			initv(cParticle->force,0,0,0);
		}

		if (forcers != NULL)	forcers->force(dtime,particles);

		for (cParticle=particles;cParticle != NULL;) {
			vector	tmp;

			cParticle->life	-=	dtime;
			if (cParticle->life <= 0) {
				CParticle	*nParticle	=	cParticle->next;
				if (cParticle==particles) {
					particles	=	cParticle->next;
				}

				cParticle->next			=	freeParticles;
				freeParticles			=	cParticle;
				cParticle				=	nParticle;
			} else {
				mulvf(tmp,cParticle->speed,dtime);
				addvv(cParticle->position,tmp);

				mulvf(tmp,cParticle->force,dtime/cParticle->mass);
				addvv(cParticle->speed,tmp);

				/*
				if (cParticle->position[COMP_Z] < -20) {
					cParticle->position[COMP_Z]	=	-20 -20 -  cParticle->position[COMP_Z];
					cParticle->speed[COMP_Z]	*=	-1;
				}
				*/

				cParticle	=	cParticle->next;
			}
		}
	} else if (integration == INTEGRATE_RK4) {
		for (cParticle=particles;cParticle != NULL;) {
			vector	tmp;
			CSink	*cSink	=	(CSink *) forcers;
			float	IK[6];
			float	K1[6];
			float	K2[6];
			float	K3[6];
			float	K4[6];
			float	CK[6];

#define	F(K,h,p,s)										\
			cSink->computeF(tmp,p,s,cParticle->mass);	\
			K[0]	=	(s)[COMP_X]*h;					\
			K[1]	=	(s)[COMP_Y]*h;					\
			K[2]	=	(s)[COMP_Z]*h;					\
			K[3]	=	tmp[COMP_X]*h;					\
			K[4]	=	tmp[COMP_Y]*h;					\
			K[5]	=	tmp[COMP_Z]*h;

#define	initK(K)			\
			K[0]	=	0;	\
			K[1]	=	0;	\
			K[2]	=	0;	\
			K[3]	=	0;	\
			K[4]	=	0;	\
			K[5]	=	0;

#define	addK(K,OK,m)				\
			K[0]	+=	OK[0]*m;	\
			K[1]	+=	OK[1]*m;	\
			K[2]	+=	OK[2]*m;	\
			K[3]	+=	OK[3]*m;	\
			K[4]	+=	OK[4]*m;	\
			K[5]	+=	OK[5]*m;


			IK[0]	=	cParticle->position[COMP_X];
			IK[1]	=	cParticle->position[COMP_Y];
			IK[2]	=	cParticle->position[COMP_Z];
			IK[3]	=	cParticle->speed[COMP_X];
			IK[4]	=	cParticle->speed[COMP_Y];
			IK[5]	=	cParticle->speed[COMP_Z];

			// Compute stages
			F(K1,dtime,IK,IK+3);


			initK(CK);
			addK(CK,IK,1);
			addK(CK,K1,1/(float) 2);
			F(K2,dtime,CK,CK+3);

			initK(CK);
			addK(CK,IK,1);
			addK(CK,K2,1/(float) 2);
			F(K3,dtime,CK,CK+3);

			initK(CK);
			addK(CK,IK,1);
			addK(CK,K3,1);
			F(K4,dtime,CK,CK+3);

			initK(CK);
			addK(CK,IK,1);
			addK(CK,K1,1 / (float) 6);
			addK(CK,K2,1 / (float) 3);
			addK(CK,K3,1 / (float) 3);
			addK(CK,K4,1 / (float) 6);


			cParticle->position[COMP_X]	=	CK[0];
			cParticle->position[COMP_Y]	=	CK[1];
			cParticle->position[COMP_Z]	=	CK[2];
			cParticle->speed[COMP_X]	=	CK[3];
			cParticle->speed[COMP_Y]	=	CK[4];
			cParticle->speed[COMP_Z]	=	CK[5];



			cParticle->life	-=	dtime;
			if (cParticle->life <= 0) {
				CParticle	*nParticle	=	cParticle->next;
				if (cParticle==particles) {
					particles	=	cParticle->next;
				}

				cParticle->next			=	freeParticles;
				freeParticles			=	cParticle;
				cParticle				=	nParticle;
			} else {
				cParticle	=	cParticle->next;
			}
		}
	}

	if (next != NULL) {
		next->step(dtime);
	}
}

///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Method				:	draw
// Description			:	Draw the particles
// Return Value			:	-
// Comments				:
// Date last edited		:	11/16/2001
void		CParticleSystem::draw(float *toCamera,float *fromCamera,float tx,float ty,float tz,float dtime)  {
	CParticle	*cParticle;

	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDepthMask(0);

	glBindTexture(GL_TEXTURE_2D,ptexture);

	glPushMatrix();
	glLoadIdentity();

	for (cParticle=particles;cParticle != NULL;cParticle=cParticle->next) {
		vector	p;

		mulmv(p,toCamera,cParticle->position);
		p[COMP_X]	+=	tx;
		p[COMP_Y]	+=	ty;
		p[COMP_Z]	+=	tz;

		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(p[COMP_X]-1,p[COMP_Y]-1,p[COMP_Z]);
		glTexCoord2f(1,0);
		glVertex3f(p[COMP_X]+1,p[COMP_Y]-1,p[COMP_Z]);
		glTexCoord2f(1,1);
		glVertex3f(p[COMP_X]+1,p[COMP_Y]+1,p[COMP_Z]);
		glTexCoord2f(0,1);
		glVertex3f(p[COMP_X]-1,p[COMP_Y]+1,p[COMP_Z]);
		glEnd();
	}

	glPopMatrix();

	glDepthMask(1);
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);

	if (next != NULL)	next->draw(toCamera,fromCamera,tx,ty,tz,dtime);
}
