#ifndef PSYSTEM_H
#define PSYSTEM_H

#include "common/algebra.h"
#include "common/os.h"

class	CRigidBody;

///////////////////////////////////////////////////////////////////////
// Class				:	CParticle
// Description			:	Holds a particle description
// Comments				:
// Date last edited		:	11/16/2001
class	CParticle {
public:
	vector				position;
	vector				speed;
	float				mass;
	float				life;

	vector				force;

	CParticle			*next;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CEmitter
// Description			:	Emits particles
// Comments				:
// Date last edited		:	11/16/2001
class	CEmitter {
public:
							CEmitter() { }
							CEmitter(float *,float,float,float,float,float,float);
							~CEmitter();

	virtual CParticle		*emit(float,CParticle *&);
	virtual	CEmitter		*clone();

	float					particlesPerSecond;
	float					particleLifeSpan;
	float					particleMass;
	float					speed;
	float					angleVariance;
	float					speedVariance;

	vector					source;
	vector					dir;

	float					numLeftoverParticles;

	CEmitter				*next;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CSemitter
// Description			:	Emits particles
// Comments				:
// Date last edited		:	11/16/2001
class	CSemitter : public CEmitter {
public:
							CSemitter() { }
							CSemitter(float *,float,float,float);
							~CSemitter();

			CParticle		*emit(float,CParticle *&);
			CEmitter		*clone();

			int				emitted;
};


///////////////////////////////////////////////////////////////////////
// Class				:	CForceSource
// Description			:	Creates forces on particles
// Comments				:
// Date last edited		:	11/16/2001
class	CForceSource {
public:
							CForceSource() { }
							CForceSource(float *);
							~CForceSource();

	virtual	void			force(float,CParticle *);
	virtual	CForceSource	*clone();

	CForceSource			*next;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CGravity
// Description			:	Simple gravity model
// Comments				:
// Date last edited		:	11/16/2001
class	CGravity : public CForceSource {
public:
							CGravity() { }
							CGravity(float *,float);
							~CGravity();

	virtual	void			force(float,CParticle *);
	virtual	CForceSource	*clone();

	vector					dir;
	float					g;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Description			:	Swirl force generator
// Comments				:
// Date last edited		:	11/16/2001
class	CSwirl : public CForceSource {
public:
							CSwirl() { }
							CSwirl(float *,float);
							~CSwirl();

	virtual	void			force(float,CParticle *);
	virtual	CForceSource	*clone();

	vector					x,y,z,o;
	float					magnitude;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CSwirl
// Description			:	Swirl force generator
// Comments				:
// Date last edited		:	11/16/2001
class	CSink : public CForceSource {
public:
							CSink() { }
							CSink(float *,float);
							~CSink();

			void			computeF(float *,float *,float *,float);
	virtual	void			force(float,CParticle *);
	virtual	CForceSource	*clone();

	vector					src;
	float					magnitude;
};

///////////////////////////////////////////////////////////////////////
// Class				:	CParticleSystem
// Description			:	Holds a particle system
// Comments				:
// Date last edited		:	11/16/2001
class	CParticleSystem {
	typedef enum {
		INTEGRATE_FEULER,
		INTEGRATE_RK4
	} EIntegrationType;
public:
						CParticleSystem(CEmitter *,CForceSource *,float *,char *);
						~CParticleSystem();

	virtual	void		collide(CRigidBody *);
	virtual	void		step(float);
	virtual	void		draw(float *,float *,float,float,float,float);

	CEmitter			*emitters;
	CForceSource		*forcers;
	CParticle			*particles;

	CParticle			*freeParticles;

	CParticleSystem		*next;
	unsigned int		ptexture;

	FILE				*outFile;
	EIntegrationType	integration;
};



#endif