%{
#undef		alloca
#include	"mySimulation.h"
#include	"common/os.h"
#include	"psystem.h"

void	sceneerror(char *);
int		scenelex(void );					// Forward definition for stupid yacc

class CBodyState {
public:
					CBodyState();
					CBodyState(CBodyState *);
					~CBodyState();

	int				fixed;	
	float			mass;								// Is the body fixed in space ?
	matrix			transform;							// The initial transformation
	float			coefficientOfFriction,coefficientOfRestitution;

	CEmitter		*emitters;
	CForceSource	*forcers;
	CParticleSystem	*psystems;

	CBodyState	*next;
};

static	CBodyState				*currentState		=	NULL;
static	CMySimulation			*currentSimulator	=	NULL;
static	int						currentObject		=	0;
		char					inputPath[1024];
static	CArray<CGeometry *>		*currentGeometry	=	NULL;

%}
%union {
  char			string[64];
  float			real;
  int			integer;

}
%token			SCENE_SEMITTER
%token			SCENE_EMITTER
%token			SCENE_FORCER
%token			SCENE_PSYSTEM
%token			SCENE_SINK
%token			SCENE_ROTATE
%token			SCENE_TRANSLATE
%token			SCENE_SCALE
%token			SCENE_ATTACH
%token			SCENE_BODY
%token			SCENE_OPEN
%token			SCENE_CLOSE
%token			SCENE_PUSH
%token			SCENE_POP
%token			SCENE_COMMA
%token			SCENE_TRUE
%token			SCENE_FALSE
%token			SCENE_EQUALS
%token			SCENE_FIXED
%token			SCENE_MASS
%token			SCENE_SEMICOLON
%token			SCENE_LIGHT
%token			SCENE_MAXTIMESTEP
%token			SCENE_GRAVITY
%token			SCENE_SWIRL
%token			SCENE_PENETRATIONTHRESHOLD
%token			SCENE_VELOCITYDAMPER
%token			SCENE_INERTIALDAMPER
%token			SCENE_FRICTION
%token			SCENE_RESTITUTION
%token			SCENE_INPATH
%token			SCENE_REFRESHRATE
%token<real>	SCENE_FLOAT
%token<string>	SCENE_TEXT
%type<integer>	sceneBoolean
%%
start:	sceneCommands;

sceneCommands:	sceneCommand
				sceneCommands
				|
				;

sceneCommand:	SCENE_PUSH
				{
					currentState	=	new CBodyState(currentState);
				}
				|
				SCENE_POP
				{
					CBodyState	*tState;

					if (currentState == NULL)	sceneerror("Bad nesting");
					else {
						tState			=	currentState->next;
						delete currentState;
						currentState	=	tState;
					}
				}
				|
				SCENE_ROTATE
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					matrix	r,t;

					rotatem(r,$3,$5,$7,$9);
					mulmm(t,currentState->transform,r);
					movmm(currentState->transform,t);
				}
				|
				SCENE_SCALE
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					matrix	r,t;

					scalem(r,$3,$5,$7);
					mulmm(t,currentState->transform,r);
					movmm(currentState->transform,t);
				}
				|
				SCENE_TRANSLATE
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					matrix	r,t;

					translatem(r,$3,$5,$7);
					mulmm(t,currentState->transform,r);
					movmm(currentState->transform,t);
				}
				|
				SCENE_EMITTER
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					CEmitter	*emitter	=	new CEmitter(currentState->transform,$3,$5,$7,$9,$11,$13);

					emitter->next			=	currentState->emitters;
					currentState->emitters	=	emitter;
				}
				|
				SCENE_SEMITTER
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					CEmitter	*emitter	=	new CSemitter(currentState->transform,$3,$5,$7);

					emitter->next			=	currentState->emitters;
					currentState->emitters	=	emitter;
				}
				|
				SCENE_GRAVITY
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					CForceSource	*forcer	=	new CGravity(currentState->transform,$3);

					forcer->next			=	currentState->forcers;
					currentState->forcers	=	forcer;
				}
				|
				SCENE_SWIRL
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					CForceSource	*forcer	=	new CSwirl(currentState->transform,$3);

					forcer->next			=	currentState->forcers;
					currentState->forcers	=	forcer;
				}
				|
				SCENE_SINK
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					CForceSource	*forcer	=	new CSink(currentState->transform,$3);

					forcer->next			=	currentState->forcers;
					currentState->forcers	=	forcer;
				}
				|
				SCENE_PSYSTEM
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					vector	color	=	{$3,$5,$7};
					CParticleSystem	*cSystem	=	new CParticleSystem(currentState->emitters,currentState->forcers,color,NULL);

					currentSimulator->addPsystem(cSystem);

					currentState->emitters		=	NULL;
					currentState->forcers		=	NULL;
				}
				|
				SCENE_PSYSTEM
				SCENE_OPEN
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_FLOAT
				SCENE_COMMA
				SCENE_TEXT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					vector	color	=	{$3,$5,$7};
					CParticleSystem	*cSystem	=	new CParticleSystem(currentState->emitters,currentState->forcers,color,$9);

					currentSimulator->addPsystem(cSystem);

					currentState->emitters		=	NULL;
					currentState->forcers		=	NULL;
				}
				|
				SCENE_BODY
				SCENE_OPEN
				SCENE_TEXT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					char	location[OS_MAX_PATH_LENGTH];

					if (COs::locateFile(location,$3,inputPath) == TRUE) {
						CMyGeometry	*geometry	=	parseGeometry(location);

						if (geometry == NULL) {
							sceneerror("Unable to read geometry");
						} else {
							int		i;
							vector	vtmp;

							// Apply the transformation to the geometry
							for (i=0;i<geometry->numVertices;i++) {
								mulmv(vtmp,currentState->transform,geometry->vertices[i]);
								movvv(geometry->vertices[i],vtmp);
							}

							if (currentGeometry == NULL)
								currentGeometry	=	new CArray<CGeometry *>;

							(*currentGeometry)	+= geometry;

							{
								int			i;
								CRigidBody	*body	=	new CRigidBody(currentSimulator->collisionDetector,currentGeometry,(currentState->fixed ? 0 : currentState->mass));
								body->bodyIndex		=	currentObject;

								currentObject++;

								for (i=0;i<currentGeometry->getSize();i++)
									((CMyGeometry *) (*currentGeometry)[i])->prepare();

								currentSimulator->addObject(body);
							}

							currentGeometry		=	NULL;
						}
					}
				}
				|
				SCENE_ATTACH
				SCENE_OPEN
				SCENE_TEXT
				SCENE_CLOSE
				SCENE_SEMICOLON
				{
					char	location[OS_MAX_PATH_LENGTH];

					if (COs::locateFile(location,$3,inputPath) == TRUE) {
						CMyGeometry	*geometry	=	parseGeometry(location);

						if (geometry == NULL) {
							sceneerror("Unable to read geometry");
						} else {
							int		i;
							vector	vtmp;

							// Apply the transformation to the geometry
							for (i=0;i<geometry->numVertices;i++) {
								mulmv(vtmp,currentState->transform,geometry->vertices[i]);
								movvv(geometry->vertices[i],vtmp);
							}

							if (currentGeometry == NULL)
								currentGeometry	=	new CArray<CGeometry *>;

							(*currentGeometry)	+= geometry;
						}
					}
				}
				|
				sceneGlobals
				|
				sceneAssignment
				;

sceneGlobals:		SCENE_LIGHT
					SCENE_OPEN
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_CLOSE
					SCENE_SEMICOLON
					{
						CMySimulation::CLight	cLight;

						cLight.ambient[COMP_R]	=	$3;
						cLight.ambient[COMP_G]	=	$5;
						cLight.ambient[COMP_B]	=	$7;
						cLight.color[COMP_R]	=	$9;
						cLight.color[COMP_G]	=	$11;
						cLight.color[COMP_B]	=	$13;
						cLight.dir[COMP_X]		=	$15;
						cLight.dir[COMP_Y]		=	$17;
						cLight.dir[COMP_Z]		=	$19;
						cLight.w				=	$21;
					
						currentSimulator->addLight(&cLight);
					}
					|
					SCENE_MAXTIMESTEP
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentSimulator->maxTimeStep	=	$3;
					}
					|
					SCENE_PENETRATIONTHRESHOLD
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentSimulator->collisionDetector->tolerance	=	$3;
					}
					|
					SCENE_VELOCITYDAMPER
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentSimulator->velocityDamping	=	$3;
					}
					|
					SCENE_INERTIALDAMPER
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentSimulator->avelocityDamping	=	$3;
					}
					|
					SCENE_GRAVITY
					SCENE_EQUALS
					SCENE_OPEN
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_CLOSE
					SCENE_SEMICOLON
					{
						initv(currentSimulator->gravity,$4,$6,$8);
					}
					|
					SCENE_FRICTION
					SCENE_OPEN
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_CLOSE
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						int		i	=	(int) $3;
						int		j	=	(int) $5;
						float	f	=	$8;

						if (i < 0) {
							if (j >= 0) {
								for (i=0;i<currentSimulator->collisionDetector->numBodies;i++) {
									currentSimulator->collisionDetector->coefficientsOfFriction[j][i]	=	f;
									currentSimulator->collisionDetector->coefficientsOfFriction[i][j]	=	f;
								}
							}
						} else if (j < 0) {
							for (j=0;j<currentSimulator->collisionDetector->numBodies;j++) {
								currentSimulator->collisionDetector->coefficientsOfFriction[j][i]	=	f;
								currentSimulator->collisionDetector->coefficientsOfFriction[i][j]	=	f;
							}
						} else {
							currentSimulator->collisionDetector->coefficientsOfFriction[j][i]	=	f;
							currentSimulator->collisionDetector->coefficientsOfFriction[i][j]	=	f;
						}
					}
					|
					SCENE_RESTITUTION
					SCENE_OPEN
					SCENE_FLOAT
					SCENE_COMMA
					SCENE_FLOAT
					SCENE_CLOSE
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						int		i	=	(int) $3;
						int		j	=	(int) $5;
						float	f	=	$8;

						if (i < 0) {
							if (j >= 0) {
								for (i=0;i<currentSimulator->collisionDetector->numBodies;i++) {
									currentSimulator->collisionDetector->coefficientsOfRestitution[j][i]	=	f;
									currentSimulator->collisionDetector->coefficientsOfRestitution[i][j]	=	f;
								}
							}
						} else if (j < 0) {
							for (j=0;j<currentSimulator->collisionDetector->numBodies;j++) {
								currentSimulator->collisionDetector->coefficientsOfRestitution[j][i]	=	f;
								currentSimulator->collisionDetector->coefficientsOfRestitution[i][j]	=	f;
							}
						} else {
							currentSimulator->collisionDetector->coefficientsOfRestitution[j][i]	=	f;
							currentSimulator->collisionDetector->coefficientsOfRestitution[i][j]	=	f;
						}
					}
					|
					SCENE_INPATH
					SCENE_EQUALS
					SCENE_TEXT
					SCENE_SEMICOLON
					{
						scpy(inputPath,$3);
					}
					|
					SCENE_REFRESHRATE
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentSimulator->refreshRate	=	$3;
					}
					;

sceneAssignment:	SCENE_FIXED
					SCENE_EQUALS
					sceneBoolean
					SCENE_SEMICOLON
					{
						currentState->fixed	=	$3;
					}
					|
					SCENE_FRICTION
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentState->coefficientOfFriction	=	$3;
						currentSimulator->collisionDetector->defaultFriction	=	$3;
					}
					|
					SCENE_RESTITUTION
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentState->coefficientOfRestitution	=	$3;
						currentSimulator->collisionDetector->defaultRestitution	=	$3;
					}
					|
					SCENE_MASS
					SCENE_EQUALS
					SCENE_FLOAT
					SCENE_SEMICOLON
					{
						currentState->mass	=	$3;
					}
					;

sceneBoolean:	SCENE_TRUE
				{
					$$	=	1;
				}
				|
				SCENE_FALSE
				{
					$$	=	0;
				}
				|
				SCENE_FLOAT
				{
					if ($1 == 0)	$$ = 0;
					else			$$ = 1;
				}
				;

%%

#include "lex.scene.cpp"

int	sceneLineNo;
int	sceneNumErrors;

CMySimulation	*parseScene(const char *file) {
	char	location[OS_MAX_PATH_LENGTH];

	if (COs::locateFile(location,file) == TRUE) {

		yyin	=	fopen(location,"r");

		if (yyin != NULL) {
			sceneLineNo			=	1;
			sceneNumErrors		=	0;

			currentState		=	new CBodyState;
			currentSimulator	=	new CMySimulation;
			currentObject		=	0;
			sprintf(inputPath,"");
			//sprintf(inputPath,"\\okan\\scenes\\tmp667\\okan\\tmp\\rbody\\meshes:\\okan\\scenes\\tmp667\\okan\\tmp\\rbody");
			//sprintf(inputPath,"\\okan\\dev\\rbody");

			sceneparse();

			{
				CBodyState	*cState,*nState;

				for (cState = currentState;cState != NULL;) {
					nState	=	cState->next;
					delete cState;
					cState	=	nState;
				}
			}

			if (sceneNumErrors == 0) {
				currentSimulator->prepare();
				return currentSimulator;
			}
		}
	}

	if (currentSimulator != NULL)	delete currentSimulator;

	return NULL;
}

void	sceneerror(char *str) {
	sceneNumErrors++;
}

CBodyState::CBodyState() {
	fixed						=	FALSE;
	mass						=	1;
	identitym(transform);
	next						=	NULL;
	coefficientOfFriction		=	0.5f;
	coefficientOfRestitution	=	0.5f;
	emitters					=	NULL;
	forcers						=	NULL;
}

CBodyState::CBodyState(CBodyState *other) {
	fixed		=	other->fixed;
	mass		=	other->mass;
	movmm(transform,other->transform);
	next		=	other;

	emitters	=	(other->emitters	!= NULL ? other->emitters->clone() : NULL);
	forcers		=	(other->forcers		!=	NULL ? other->forcers->clone() : NULL);
}

CBodyState::~CBodyState() {
	if (emitters	!= NULL)	delete emitters;
	if (forcers		!=	NULL)	delete forcers;
}
